@@ -46,6 +46,7 @@ extern "C"
 #include <limits.h>
 #include <stdlib.h>
 #include "omc_msvc.h"
+#include "omc_file.h"
 #include "openmodelica.h"
 #include "meta_modelica.h"
 #include "ModelicaUtilities.h"
@@ -78,6 +79,21 @@ extern const char* System_stringReplace(const char* str, const char* source, con
   return res;
 }
 
+extern const char* System_makeC89Identifier(const char* str)
+{
+  int i=0, len=strlen(str);
+  char *res = omc_alloc_interface.malloc_strdup(str);
+  if (!((res[0]>='a' && res[0]<='z') || (res[0]>='A' && res[0]<='Z'))) {
+    res[0] = '_';
+  }
+  for (i=1; i<len; i++) {
+    if (!((res[i]>='a' && res[i]<='z') || (res[i]>='A' && res[i]<='Z') || (res[i]>='0' && res[i]<='9'))) {
+      res[i] = '_';
+    }
+  }
+  return res;
+}
+
 extern int System_stringFind(const char* str, const char* searchStr)
 {
   const char *found = strstr(str, searchStr);
@@ -119,21 +135,6 @@ extern int System_realtimeNtick(int ix)
   return rt_ncall(ix);
 }
 
-extern const char* System_getRTLibs()
-{
-  return LDFLAGS_RT;
-}
-
-extern const char* System_getRTLibsSim()
-{
-  return LDFLAGS_RT_SIM;
-}
-
-extern const char* System_getRTLibsFMU()
-{
-  return LDFLAGS_RT_SOURCE_FMU;
-}
-
 extern const char* System_getCCompiler()
 {
   return cc;
@@ -164,21 +165,6 @@ extern const char* System_getCFlags()
   return cflags;
 }
 
-extern const char* System_getExeExt()
-{
-  return CONFIG_EXE_EXT;
-}
-
-extern const char* System_getDllExt()
-{
-  return CONFIG_DLL_EXT;
-}
-
-extern const char* System_os()
-{
-  return CONFIG_OS;
-}
-
 extern const char* System_trim(const char* str, const char* chars_to_remove)
 {
   return SystemImpl__trim(str,chars_to_remove);
@@ -217,26 +203,78 @@ extern const char* System_dirname(const char* str)
   return res;
 }
 
+#if defined(OPENMODELICA_BOOTSTRAPPING_STAGE_1)
+extern const char* System_getRTLibs()
+{
+  return "DUMMY RT LIBS";
+}
+
+extern const char* System_getRTLibsSim()
+{
+  return "DUMMY RT LIBS";
+}
+
+extern const char* System_getRTLibsFMU()
+{
+  return "DUMMY RT LIBS";
+}
+
+extern const char* System_getExeExt()
+{
+  return ".exe";
+}
+
+extern const char* System_getDllExt()
+{
+  return CONFIG_DLL_EXT;
+}
+
+extern const char* System_getMakeCommand()
+{
+  return "make";
+}
+
+extern const char* System_os()
+{
+  return "Windows_NT";
+}
+
 extern const char* System_configureCommandLine()
 {
-  return CONFIGURE_COMMANDLINE;
+  return "Dummy configure";
 }
 
 extern const char* System_platform()
 {
-  return CONFIG_PLATFORM;
+  return "Dummy platform";
 }
 
 extern const char* System_pathDelimiter()
 {
-  return CONFIG_PATH_DELIMITER;
+  return "/";
 }
 
 extern const char* System_groupDelimiter()
 {
-  return CONFIG_GROUP_DELIMITER;
+  return ";";
 }
 
+extern const char* System_getCorbaLibs()
+{
+  return "Dummy CORBA libs";
+}
+
+extern void* System_getRuntimeLibs()
+{
+  return "Dummy system libs";
+}
+
+const char* System_getTriple()
+{
+  return "DEFAULT_TRIPLE";
+}
+#endif
+
 extern int System_strncmp(const char *str1, const char *str2, int len)
 {
   int res= strncmp(str1,str2,len);
@@ -274,6 +312,16 @@ extern void System_setHasExpandableConnectors(int b)
   hasExpandableConnectors = b;
 }
 
+extern int System_getHasOverconstrainedConnectors()
+{
+  return hasOverconstrainedConnectors;
+}
+
+extern void System_setHasOverconstrainedConnectors(int b)
+{
+  hasOverconstrainedConnectors = b;
+}
+
 extern int System_getPartialInstantiation()
 {
   return isPartialInstantiation;
@@ -460,6 +508,47 @@ void* System_moFiles(const char *directory)
 }
 #endif
 
+#if defined(__MINGW32__) || defined(_MSC_VER)
+void* System_mocFiles(const char *directory)
+{
+  void *res;
+  WIN32_FIND_DATA FileData;
+  BOOL more = TRUE;
+  char pattern[1024];
+  HANDLE sh;
+  sprintf(pattern, "%s\\*.moc", directory);
+  res = mmc_mk_nil();
+  sh = FindFirstFile(pattern, &FileData);
+  if (sh != INVALID_HANDLE_VALUE) {
+    while(more) {
+      if (strcmp(FileData.cFileName,"package.moc") != 0)
+      {
+        res = mmc_mk_cons(mmc_mk_scon(FileData.cFileName),res);
+      }
+      more = FindNextFile(sh, &FileData);
+    }
+    if (sh != INVALID_HANDLE_VALUE) FindClose(sh);
+  }
+  return res;
+}
+#else
+void* System_mocFiles(const char *directory)
+{
+  int i,count;
+  void *res;
+  struct dirent **files;
+  select_from_dir = directory;
+  count = scandir(directory, &files, file_select_moc, NULL);
+  res = mmc_mk_nil();
+  for (i=0; i<count; i++)
+  {
+    res = mmc_mk_cons(mmc_mk_scon(files[i]->d_name),res);
+    free(files[i]);
+  }
+  return res;
+}
+#endif
+
 extern int System_lookupFunction(int _inLibHandle, const char* _inFunc)
 {
   int res = SystemImpl__lookupFunction(_inLibHandle, _inFunc);
@@ -482,6 +571,15 @@ extern int System_userIsRoot()
   return CONFIG_USER_IS_ROOT;
 }
 
+extern int System_getuid()
+{
+#if defined(__MINGW32__) || defined(_MSC_VER)
+  return 0;
+#else
+  return getuid();
+#endif
+}
+
 extern const char* System_readEnv(const char *envname)
 {
   char *envvalue = getenv(envname);
@@ -561,16 +659,6 @@ void* System_subDirectories(const char *directory)
 }
 #endif
 
-extern const char* System_getCorbaLibs()
-{
-  return CONFIG_CORBALIBS;
-}
-
-extern void* System_getRuntimeLibs()
-{
-  return CONFIG_SYSTEMLIBS;
-}
-
 extern void* System_regex(const char* str, const char* re, int maxn, int extended, int sensitive, int *nmatch)
 {
   void *res;
@@ -703,11 +791,6 @@ extern void System_getGCStatus(double *used, double *allocated)
   *used = *allocated - GC_get_free_bytes();
 }
 
-extern const char* System_getMakeCommand()
-{
-  return DEFAULT_MAKE;
-}
-
 extern const char* System_snprintff(const char *fmt, int len, double d)
 {
   char *buf;
@@ -721,13 +804,60 @@ extern const char* System_snprintff(const char *fmt, int len, double d)
   return buf;
 }
 
+extern const char* System_sprintff(const char *fmt, double d)
+{
+  char *buf;
+  const int buf_size = 20;
+  buf = ModelicaAllocateString(buf_size);
+
+  int len = snprintf(buf, buf_size, fmt, d);
+
+  if (len < 0) {
+    MMC_THROW();
+  }
+
+  if (len >= buf_size) {
+    buf = ModelicaAllocateString(len + 1);
+    snprintf(buf, len, fmt, d);
+  }
+
+  return buf;
+}
+
 extern const char* System_realpath(const char *path)
 {
+#if defined(__MINGW32__) || defined(_MSC_VER)
+  MULTIBYTE_TO_WIDECHAR_LENGTH(path, unicodePathLength);
+  MULTIBYTE_TO_WIDECHAR_VAR(path, unicodePath, unicodePathLength);
+
+  DWORD bufLen = 0;
+  bufLen = GetFullPathNameW(unicodePath, bufLen, NULL, NULL);
+  if (!bufLen) {
+    fprintf(stderr, "GetFullPathNameW failed. %lu\n", GetLastError());
+    MMC_THROW();
+  }
+
+  WCHAR unicodeFullPath[bufLen];
+  if (!GetFullPathNameW(unicodePath, bufLen, unicodeFullPath, NULL)) {
+    MULTIBYTE_OR_WIDECHAR_VAR_FREE(unicodePath);
+    fprintf(stderr, "GetFullPathNameW failed. %lu\n", GetLastError());
+    MMC_THROW();
+  }
+  MULTIBYTE_OR_WIDECHAR_VAR_FREE(unicodePath);
+
+  WIDECHAR_TO_MULTIBYTE_LENGTH(unicodeFullPath, bufferLength);
+  WIDECHAR_TO_MULTIBYTE_VAR(unicodeFullPath, buffer, bufferLength);
+  SystemImpl__toWindowsSeperators(buffer, bufferLength);
+  char *res = omc_alloc_interface.malloc_strdup(buffer);
+  MULTIBYTE_OR_WIDECHAR_VAR_FREE(buffer);
+  return res;
+#else
   char buf[PATH_MAX];
   if (realpath(path, buf) == NULL) {
     MMC_THROW();
   }
   return omc_alloc_interface.malloc_strdup(buf);
+#endif
 }
 
 extern int System_fileIsNewerThan(const char *file1, const char *file2)
@@ -800,8 +930,39 @@ extern void* System_launchParallelTasks(threadData_t *threadData, int numThreads
 #if defined(__MINGW32__)
   /* adrpo: set thread stack size on Windows to 4MB */
   pthread_attr_t attr;
-  pthread_attr_init(&attr);
-  pthread_attr_setstacksize(&attr, 4194304);
+  if (pthread_attr_init(&attr))
+  {
+    const char *tok[1] = {strerror(errno)};
+    data.fail = 1;
+    c_add_message(NULL,5999,
+      ErrorType_scripting,
+      ErrorLevel_internal,
+      gettext("System.launchParallelTasks: failed to initialize the pthread attributes: %s"),
+      NULL,
+      0);
+    MMC_THROW_INTERNAL();
+  }
+  /* try to set a stack size of 4MB */
+  if (pthread_attr_setstacksize(&attr, 4194304))
+  {
+    /* did not work, try half 2MB */
+    if (pthread_attr_setstacksize(&attr, 2097152))
+    {
+      /* did not work, try half 1MB */
+      if (pthread_attr_setstacksize(&attr, 1048576))
+      {
+        const char *tok[1] = {strerror(errno)};
+        data.fail = 1;
+        c_add_message(NULL,5999,
+          ErrorType_scripting,
+          ErrorLevel_internal,
+          gettext("System.launchParallelTasks: failed to set the pthread stack size to 1MB: %s"),
+          NULL,
+          0);
+        MMC_THROW_INTERNAL();
+      }
+    }
+  }
 #endif
 
 #else /* MSVC */
