@@ -51,8 +51,6 @@ extern "C" {
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
-#include <sys/stat.h>
-#include <sys/types.h>
 #include <time.h>
 #include <math.h>
 
@@ -106,16 +104,6 @@ typedef void* iconv_t;
 #define S_IFLNK  0120000  /* symbolic link */
 
 #include <sys/types.h>
-#if defined(_MSC_VER)
-  #include <win32_dirent.h>
-  #if !defined(PATH_MAX)
-  #define PATH_MAX MAX_PATH
-  #endif
-#else
-  #include <dirent.h>
-#endif
-
-char *realpath(const char *path, char resolved_path[PATH_MAX]);
 
 #else
 /* includes/defines specific for LINUX/OS X */
@@ -170,7 +158,11 @@ static char *linker = (char *)def_linker;
 static char *cflags = (char *)def_cflags;
 static char *ldflags= (char *)def_ldflags;
 
+/* TODO! FIXME!
+ * we need to move these to threadData if we are to run things in parallel in OMC!
+ */
 static int hasExpandableConnectors = 0;
+static int hasOverconstrainedConnectors = 0;
 static int hasInnerOuterDefinitions = 0;
 static int hasStreamConnectors = 0;
 static int isPartialInstantiation = 0;
@@ -267,25 +259,64 @@ extern int SystemImpl__setLDFlags(const char *str)
   return 0;
 }
 
+#if defined(__MINGW32__) || defined(_MSC_VER)
+/* Make sure windows paths use frontslash and not backslash */
+void SystemImpl__toWindowsSeperators(char* buffer, int bufferLength)
+{
+  int i;
+  for (i=0; i<bufferLength && buffer[i]; i++) {
+    if (buffer[i] == '\\') buffer[i] = '/';
+  }
+}
+#endif
+
+int SystemImpl__chdir(const char* path)
+{
+#if defined(__MINGW32__) || defined(_MSC_VER)
+  MULTIBYTE_TO_WIDECHAR_LENGTH(path, unicodePathLength);
+  MULTIBYTE_TO_WIDECHAR_VAR(path, unicodePath, unicodePathLength);
+
+  if (!SetCurrentDirectoryW(unicodePath)) {
+    MULTIBYTE_OR_WIDECHAR_VAR_FREE(unicodePath);
+    c_add_message(NULL,-1,ErrorType_scripting,ErrorLevel_error,gettext("SetCurrentDirectoryW failed."),NULL,0);
+    return -1;
+  }
+  MULTIBYTE_OR_WIDECHAR_VAR_FREE(unicodePath);
+  return 0;
+#else
+  if (chdir(path) != 0) {
+    c_add_message(NULL,-1,ErrorType_scripting,ErrorLevel_error,gettext("chdir failed."),NULL,0);
+    return -1;
+  }
+  return 0;
+#endif
+}
+
 extern char* SystemImpl__pwd(void)
 {
-  char buf[MAXPATHLEN];
 #if defined(__MINGW32__) || defined(_MSC_VER)
-  int i;
-  LPTSTR bufPtr=buf;
-  DWORD bufLen = MAXPATHLEN;
-  if (!GetCurrentDirectory(bufLen,bufPtr)) {
-    fprintf(stderr, "System.pwd failed\n");
+  DWORD bufLen = 0;
+  bufLen = GetCurrentDirectoryW(bufLen, 0);
+  if (!bufLen) {
+    c_add_message(NULL,-1,ErrorType_scripting,ErrorLevel_error,gettext("GetCurrentDirectoryW failed."),NULL,0);
     return NULL;
   }
-  /* Make sure windows paths use frontslash and not backslash */
-  for (i=0; i<MAXPATHLEN && buf[i]; i++) {
-    if (buf[i] == '\\') buf[i] = '/';
+
+  WCHAR unicodePath[bufLen];
+  if (!GetCurrentDirectoryW(bufLen, unicodePath)) {
+    c_add_message(NULL,-1,ErrorType_scripting,ErrorLevel_error,gettext("GetCurrentDirectoryW failed."),NULL,0);
+    return NULL;
   }
-  return omc_alloc_interface.malloc_strdup(buf);
+  WIDECHAR_TO_MULTIBYTE_LENGTH(unicodePath, bufferLength);
+  WIDECHAR_TO_MULTIBYTE_VAR(unicodePath, buffer, bufferLength);
+  SystemImpl__toWindowsSeperators(buffer, bufferLength);
+  char *res = omc_alloc_interface.malloc_strdup(buffer);
+  MULTIBYTE_OR_WIDECHAR_VAR_FREE(buffer);
+  return res;
 #else
+  char buf[MAXPATHLEN];
   if (NULL == getcwd(buf,MAXPATHLEN)) {
-    fprintf(stderr, "System.pwd failed\n");
+    c_add_message(NULL,-1,ErrorType_scripting,ErrorLevel_error,gettext("System.pwd failed."),NULL,0);
     return NULL;
   }
   return omc_alloc_interface.malloc_strdup(buf);
@@ -294,11 +325,16 @@ extern char* SystemImpl__pwd(void)
 
 extern int SystemImpl__regularFileExists(const char* str)
 {
-#if defined(_MSC_VER)
-  WIN32_FIND_DATA FileData;
+#if defined(__MINGW32__) || defined(_MSC_VER)
+  WIN32_FIND_DATAW FileData;
   HANDLE sh;
 
-  sh = FindFirstFile(str, &FileData);
+  MULTIBYTE_TO_WIDECHAR_LENGTH(str, unicodeFilenameLength);
+  MULTIBYTE_TO_WIDECHAR_VAR(str, unicodeFilename, unicodeFilenameLength);
+
+  sh = FindFirstFileW(unicodeFilename, &FileData);
+
+  MULTIBYTE_OR_WIDECHAR_VAR_FREE(unicodeFilename);
 
   if (sh == INVALID_HANDLE_VALUE) {
     if (strlen(str) >= MAXPATHLEN)
@@ -328,7 +364,7 @@ extern int SystemImpl__regularFileWritable(const char* str)
   FILE *f;
   if (!SystemImpl__regularFileExists(str))
     return 0;
-  f = fopen(str, "a");
+  f = omc_fopen(str, "a");
   if (f == NULL)
     return 0;
   fclose(f);
@@ -340,8 +376,12 @@ static char* SystemImpl__readFile(const char* filename)
   char* buf;
   int res;
   FILE * file = NULL;
+#if defined(__MINGW32__) || defined(_MSC_VER)
+  struct _stat statstr;
+#else
   struct stat statstr;
-  res = stat(filename, &statstr);
+#endif
+  res = omc_stat(filename, &statstr);
 
   if (res != 0) {
     const char *c_tokens[2]={strerror(errno),filename};
@@ -355,7 +395,7 @@ static char* SystemImpl__readFile(const char* filename)
   }
 
   /* adrpo: if size is larger than the max string, return a different string */
-#ifndef _LP64
+#if !(defined(_LP64) || defined(_LLP64) || defined(_WIN64) || defined(__MINGW64__))
   if (statstr.st_size > (pow((double)2, (double)22) * 4)) {
     const char *c_tokens[1]={filename};
     c_add_message(NULL,85, /* ERROR_OPENING_FILE */
@@ -368,7 +408,7 @@ static char* SystemImpl__readFile(const char* filename)
   }
 #endif
 
-  file = fopen(filename,"rb");
+  file = omc_fopen(filename,"rb");
   if (file == NULL) {
     const char *c_tokens[2]={strerror(errno),filename};
     c_add_message(NULL, 85, /* ERROR_OPENING_FILE */
@@ -400,11 +440,7 @@ static char* SystemImpl__readFile(const char* filename)
 /* returns 0 on success */
 int SystemImpl__removeFile(const char* filename)
 {
-#if defined(__MINGW32__) || defined(_MSC_VER)
-  return _unlink(filename);
-#else /* unix */
-  return unlink(filename);
-#endif
+  return omc_unlink(filename);
 }
 
 /* returns 0 on success */
@@ -418,10 +454,10 @@ int SystemImpl__writeFile(const char* filename, const char* data)
   FILE * file = NULL;
   int len = strlen(data); /* MMC_HDRSTRLEN(MMC_GETHDR(rmlA1)); */
 #if defined(__APPLE_CC__)||defined(__MINGW32__)||defined(__MINGW64__)
-  unlink(filename);
+  SystemImpl__removeFile(filename);
 #endif
   /* adrpo: 2010-09-22 open the file in BINARY mode as otherwise \r\n becomes \r\r\n! */
-  file = fopen(filename,fileOpenMode);
+  file = omc_fopen(filename,fileOpenMode);
   if (file == NULL) {
     const char *c_tokens[1]={filename};
     c_add_message(NULL,21, /* WRITING_FILE_ERROR */
@@ -551,10 +587,10 @@ const char* SystemImpl__basename(const char *str)
 #if defined(__MINGW32__) || defined(_MSC_VER)
 int runProcess(const char* cmd)
 {
-  STARTUPINFO si;
+  STARTUPINFOW si;
   PROCESS_INFORMATION pi;
   char *c = "cmd /c";
-  char *command = (char *)omc_alloc_interface.malloc_atomic(strlen(cmd) + strlen(c) + 2);
+  char *command = (char *)omc_alloc_interface.malloc_atomic(strlen(cmd) + strlen(c) + 4);
   DWORD exitCode = 1;
 
   ZeroMemory(&si, sizeof(si));
@@ -562,18 +598,22 @@ int runProcess(const char* cmd)
   ZeroMemory(&pi, sizeof(pi));
 
 
-  sprintf(command, "%s %s", c, cmd);
+  sprintf(command, "%s \"%s\"", c, cmd);
 
   /* fprintf(stderr, "%s\n", command); fflush(NULL); */
 
-  if (CreateProcessA(NULL, command, NULL, NULL, FALSE, CREATE_NO_WINDOW, NULL, NULL, &si, &pi))
+  MULTIBYTE_TO_WIDECHAR_LENGTH(command, unicodeCommandLength);
+  MULTIBYTE_TO_WIDECHAR_VAR(command, unicodeCommand, unicodeCommandLength);
+
+  if (CreateProcessW(NULL, unicodeCommand, NULL, NULL, FALSE, CREATE_NO_WINDOW, NULL, NULL, &si, &pi))
   {
-      WaitForSingleObject(pi.hProcess, INFINITE);
-      // Get the exit code.
-      GetExitCodeProcess(pi.hProcess, &exitCode);
-      CloseHandle(pi.hProcess);
-      CloseHandle(pi.hThread);
+    WaitForSingleObject(pi.hProcess, INFINITE);
+    // Get the exit code.
+    GetExitCodeProcess(pi.hProcess, &exitCode);
+    CloseHandle(pi.hProcess);
+    CloseHandle(pi.hThread);
   }
+  MULTIBYTE_OR_WIDECHAR_VAR_FREE(unicodeCommand);
   GC_free(command);
   return (int)exitCode;
 }
@@ -603,6 +643,9 @@ int SystemImpl__systemCall(const char* str, const char* outFile)
     if (*outFile) {
       /* redirect stdout, stderr in the fork'ed process */
       int fd = open(outFile, O_RDWR | O_CREAT, S_IRUSR | S_IWUSR);
+      if (fd < 0) {
+        _exit(1);
+      }
       dup2(fd, 1);
       dup2(fd, 2);
 #if defined(__APPLE_CC__)
@@ -698,13 +741,13 @@ char* System_popen(threadData_t *threadData, const char* command, int *status)
 #endif
 }
 
-#ifdef WITH_HWLOC
+#if WITH_HWLOC==1
 #include <hwloc.h>
 #endif
 
 int System_numProcessors(void)
 {
-#ifdef WITH_HWLOC
+#if WITH_HWLOC==1
   hwloc_topology_t topology;
   if (0==hwloc_topology_init(&topology) && 0==hwloc_topology_load(topology)) {
     int depth = hwloc_get_type_depth(topology, HWLOC_OBJ_CORE);
@@ -896,6 +939,61 @@ extern int SystemImpl__createDirectory(const char *str)
   }
 }
 
+extern int SystemImpl__copyFile(const char *str_1, const char *str_2)
+{
+  int rv = 1;
+  size_t n;
+  char buf[8192];
+  FILE *source, *target;
+
+  source = fopen(str_1, "r");
+  if (source==0) {
+    const char *msg[2] = {strerror(errno), str_1};
+    c_add_message(NULL,85,
+      ErrorType_scripting,
+      ErrorLevel_error,
+      gettext("Error opening file for reading %s: %s"),
+      msg,
+      2);
+    return 0;
+  }
+  target = fopen(str_2, "w");
+  if (target==0) {
+    const char *msg[2] = {strerror(errno), str_2};
+    c_add_message(NULL,85,
+      ErrorType_scripting,
+      ErrorLevel_error,
+      gettext("Error opening file for writing %s: %s"),
+      msg,
+      2);
+    fclose(source);
+    return 0;
+  }
+
+  while ( n = fread(buf, 1, 8192, source) ) {
+    if (n != fwrite(buf, 1, n, target)) {
+      rv = 0;
+      break;
+    }
+  }
+  if (rv == 0) {
+    const char *msg[3] = {strerror(errno), str_2, str_1};
+    c_add_message(NULL,85,
+      ErrorType_scripting,
+      ErrorLevel_error,
+      gettext("Error copying file contents %s to %s: %s"),
+      msg,
+      3);
+  }
+  if (!feof(source)) {
+    rv = 0;
+  }
+
+  fclose(source);
+  fclose(target);
+  return rv;
+}
+
 static char * SystemImpl__NextDir(const char * path)
 {
   char * res = NULL;
@@ -1118,13 +1216,17 @@ extern int SystemImpl__removeDirectory(const char *path)
   return retval==0;
 }
 
-extern char* SystemImpl__readFileNoNumeric(const char* filename)
+extern const char* SystemImpl__readFileNoNumeric(const char* filename)
 {
   char* buf, *bufRes;
   int res,numCount;
   FILE * file = NULL;
+#if defined(__MINGW32__) || defined(_MSC_VER)
+  struct _stat statstr;
+#else
   struct stat statstr;
-  res = stat(filename, &statstr);
+#endif
+  res = omc_stat(filename, &statstr);
 
   if(res!=0) {
     const char *c_tokens[1]={filename};
@@ -1137,7 +1239,7 @@ extern char* SystemImpl__readFileNoNumeric(const char* filename)
     return "No such file";
   }
 
-  file = fopen(filename,"rb");
+  file = omc_fopen(filename,"rb");
   buf = (char*) omc_alloc_interface.malloc_atomic(statstr.st_size+1);
   bufRes = (char*) omc_alloc_interface.malloc_atomic((statstr.st_size+70)*sizeof(char));
   if( (res = fread(buf, sizeof(char), statstr.st_size, file)) != statstr.st_size) {
@@ -1180,6 +1282,24 @@ static int file_select_mo(direntry entry)
   }
 }
 
+static int file_select_moc(direntry entry)
+{
+  char* ptr;
+  if ((strcmp(entry->d_name, ".") == 0) ||
+      (strcmp(entry->d_name, "..") == 0) ||
+      (strcmp(entry->d_name, "package.moc") == 0)) {
+    return (0);
+  } else {
+    ptr = (char*)rindex(entry->d_name, '.');
+    if ((ptr != NULL) &&
+  ((strcmp(ptr, ".moc") == 0))) {
+      return (1);
+    } else {
+      return (0);
+    }
+  }
+}
+
 #endif
 
 #if defined(__MINGW32__) || defined(_MSC_VER)
@@ -1600,20 +1720,19 @@ extern char* SystemImpl__unescapedString(const char* str)
   return res;
 }
 
-extern char* SystemImpl__unquoteIdentifier(const char* str)
+extern char* System_forceQuotedIdentifier(const char* str)
 {
   const char lookupTbl[] = {'0','1','2','3','4','5','6','7','8','9','A','B','C','D','E','F'};
   char *res,*cur;
   int len,i;
   const int offset = 10;
   const char _omcQuot[]="_omcQuot_";
-  if (*str != '\'') return NULL;
-  len = strlen(str)-2;
+  len = strlen(str);
   res = (char*) omc_alloc_interface.malloc_atomic(2*len+offset+64);
   cur = res;
   cur += sprintf(cur,"%s",_omcQuot);
   for (i=0; i<len; i++) {
-    unsigned char c = str[i+1];
+    unsigned char c = str[i];
     *cur = lookupTbl[c/16];
     cur++;
     *cur = lookupTbl[c%16];
@@ -1623,6 +1742,20 @@ extern char* SystemImpl__unquoteIdentifier(const char* str)
   return res;
 }
 
+extern char* SystemImpl__unquoteIdentifier(const char* str)
+{
+  char c = *str;
+  if ((c == '\'')
+#if !defined(OPENMODELICA_BOOTSTRAPPING_STAGE_1)
+   || strstr(str, "$")
+#endif
+  )
+  {
+    return System_forceQuotedIdentifier(str);
+  }
+  return NULL;
+}
+
 #define TIMER_MAX_STACK  1000
 static double timerIntervalTime = 0;
 static double timerCummulatedTime = 0;
@@ -1916,13 +2049,13 @@ static modelicaPathEntry* getAllModelicaPaths(const char *name, size_t nlen, voi
 #else
         mightbedir = 1;
 #endif
-        if (mightbedir && regularFileExistsInDirectory(mp,ent->d_name,"package.mo")) {
+        if (mightbedir && (regularFileExistsInDirectory(mp,ent->d_name,"package.mo") || regularFileExistsInDirectory(mp,ent->d_name,"package.moc"))) {
           /* fprintf(stderr, "found match %d %s\n", *numMatches, ent->d_name); */
           (*numMatches)++;
           continue;
         }
         entlen = strlen(ent->d_name);
-        if (entlen > 3 && 0==strcmp(ent->d_name+entlen-3,".mo") && regularFileExistsInDirectory(mp,"",ent->d_name)) {
+        if (((entlen > 3 && 0==strcmp(ent->d_name+entlen-3,".mo")) || (entlen > 4 && 0==strcmp(ent->d_name+entlen-4,".moc"))) && regularFileExistsInDirectory(mp,"",ent->d_name)) {
           /* fprintf(stderr, "found match %d %s\n", *numMatches, ent->d_name); */
           (*numMatches)++;
         }
@@ -1948,13 +2081,13 @@ static modelicaPathEntry* getAllModelicaPaths(const char *name, size_t nlen, voi
 #else
         maybeDir = 1;
 #endif
-        if (maybeDir && regularFileExistsInDirectory(mp,ent->d_name,"package.mo")) {
+        if (maybeDir && (regularFileExistsInDirectory(mp,ent->d_name,"package.mo") || regularFileExistsInDirectory(mp,ent->d_name,"package.moc"))) {
           ok=1;
           res[i].fileIsDir=1;
           /* fprintf(stderr, "found dir match: %ld %s - ok=%d\n", i, ent->d_name, ok); */
         }
         entlen = strlen(ent->d_name);
-        if (!ok && entlen > 3 && 0==strcmp(ent->d_name+entlen-3,".mo") && regularFileExistsInDirectory(mp,"",ent->d_name)) {
+        if (!ok && ((entlen > 3 && 0==strcmp(ent->d_name+entlen-3,".mo")) || (entlen > 4 && 0==strcmp(ent->d_name+entlen-4,".moc"))) && regularFileExistsInDirectory(mp,"",ent->d_name)) {
           /* fprintf(stderr, "found match file: %ld %s - ok=%d\n", i, ent->d_name, ok); */
           res[i].fileIsDir=0;
           ok=1;
@@ -2229,7 +2362,7 @@ extern int SystemImpl__reopenStandardStream(int id,const char *filename)
   return 1;
 }
 
-char* SystemImpl__iconv__ascii(const char * str)
+const char* SystemImpl__iconv__ascii(const char * str)
 {
   char *buf = 0;
   size_t sz;
@@ -2247,7 +2380,7 @@ static int isUtf8Encoding(const char *str)
   return strcasecmp(str, "UTF-8") || strcasecmp(str, "UTF8");
 }
 
-extern char* SystemImpl__iconv(const char * str, const char *from, const char *to, int printError)
+extern const char* SystemImpl__iconv(const char * str, const char *from, const char *to, int printError)
 {
   char *in_str,*res=NULL;
   size_t sz,out_sz,buflen;
@@ -2257,31 +2390,31 @@ extern char* SystemImpl__iconv(const char * str, const char *from, const char *t
   sz = strlen(str);
   if (isUtf8Encoding(from) && isUtf8Encoding(to))
   {
-    is_utf8(str, sz, &res, &count);
+    is_utf8((unsigned char*)str, sz, &res, &count);
     if (res==NULL) {
       /* Converting from UTF-8 to UTF-8 and the sequence is already UTF-8... */
       return str;
     }
     /* Converting from UTF-8, but is not valid UTF-8. Just quit early. */
     if (printError) {
-      char *ignore = SystemImpl__iconv__ascii(str);
+      const char *ignore = SystemImpl__iconv__ascii(str);
       const char *tokens[4] = {res,from,to,ignore};
-      c_add_message(NULL,-1,ErrorType_scripting,ErrorLevel_error,gettext("iconv(\"%s\",to=\"%s\",from=\"%s\") failed: %s"),tokens,4);
-      omc_alloc_interface.free_uncollectable(ignore);
+      c_add_message(NULL,-1,ErrorType_scripting,ErrorLevel_error,gettext("iconv(\"%s\",from=\"%s\",to=\"%s\") failed: %s"),tokens,4);
+      omc_alloc_interface.free_uncollectable((char*)ignore);
     }
-    return (char*) "";
+    return (const char*) "";
   }
   buflen = sz*4;
   /* fprintf(stderr,"iconv(%s,to=%s,%s) of size %d, buflen %d\n",str,to,from,sz,buflen); */
   ic = iconv_open(to, from);
   if (ic == (iconv_t) -1) {
     if (printError) {
-      char *ignore = SystemImpl__iconv__ascii(str);
+      const char *ignore = SystemImpl__iconv__ascii(str);
       const char *tokens[4] = {strerror(errno),from,to,ignore};
       c_add_message(NULL,-1,ErrorType_scripting,ErrorLevel_error,gettext("iconv(\"%s\",to=\"%s\",from=\"%s\") failed: %s"),tokens,4);
-      omc_alloc_interface.free_uncollectable(ignore);
+      omc_alloc_interface.free_uncollectable((char*)ignore);
     }
-    return (char*) "";
+    return (const char*) "";
   }
   buf = (char*) omc_alloc_interface.malloc_atomic(buflen);
   if (0 == buf) {
@@ -2289,7 +2422,7 @@ extern char* SystemImpl__iconv(const char * str, const char *from, const char *t
       /* Make the error message small so we perhaps have a chance to recover */
       c_add_message(NULL,-1,ErrorType_scripting,ErrorLevel_error,gettext("iconv() ran out of memory"),NULL,0);
     }
-    return (char*) "";
+    return (const char*) "";
   }
   *buf = 0;
   in_str = (char*) str;
@@ -2299,24 +2432,24 @@ extern char* SystemImpl__iconv(const char * str, const char *from, const char *t
   iconv_close(ic);
   if (count == -1) {
     if (printError) {
-      char *ignore = SystemImpl__iconv__ascii(str);
+      const char *ignore = SystemImpl__iconv__ascii(str);
       const char *tokens[4] = {strerror(errno),from,to,ignore};
       c_add_message(NULL,-1,ErrorType_scripting,ErrorLevel_error,gettext("iconv(\"%s\",to=\"%s\",from=\"%s\") failed: %s"),tokens,4);
-      omc_alloc_interface.free_uncollectable(ignore);
+      omc_alloc_interface.free_uncollectable((char*)ignore);
     }
     omc_alloc_interface.free_uncollectable(buf);
-    return (char*) "";
+    return (const char*) "";
   }
   buf[(buflen-1)-out_sz] = 0;
   if (strlen(buf) != (buflen-1)-out_sz) {
     if (printError) c_add_message(NULL,-1,ErrorType_scripting,ErrorLevel_error,gettext("iconv(to=%s) failed because the character set output null bytes in the middle of the string."),&to,1);
     omc_alloc_interface.free_uncollectable(buf);
-    return (char*) "";
+    return (const char*) "";
   }
   if (!strcmp(from, to) && !strcmp(str, buf))
   {
     omc_alloc_interface.free_uncollectable(buf);
-    return (char*)str;
+    return (const char*)str;
   }
   return buf;
 }
@@ -2452,184 +2585,6 @@ const char* SystemImpl__gettext(const char *msgid)
 }
 
 
-#if defined(__MINGW32__) || defined(_MSC_VER)
-
-#ifdef __MINGW32__
-
-char *realpath(const char *path, char resolved_path[PATH_MAX])
-{
-  if (!_fullpath(resolved_path, path, PATH_MAX))
-  {
-    const char *c_tokens[0]={};
-    const char* fmt = "System.realpath failed on %s with errno: %d";
-    char* msg = (char*)omc_alloc_interface.malloc_atomic(strlen(path) + strlen(fmt) + 10);
-    sprintf(msg, fmt, path, errno);
-    c_add_message(NULL,6000,
-      ErrorType_scripting,
-      ErrorLevel_warning,
-      msg,
-      c_tokens,
-      0);
-    resolved_path = (char*)path;
-  }
-  return resolved_path;
-}
-
-#else
-
-/*
-realpath() Win32 implementation, supports non standard glibc extension
-This file has no copyright assigned and is placed in the Public Domain.
-Written by Nach M. S. September 8, 2005
-*/
-
-#include <windows.h>
-#include <stdlib.h>
-#include <limits.h>
-#include <errno.h>
-#include <sys/stat.h>
-
-char *realpath(const char *path, char resolved_path[PATH_MAX])
-{
-  char *return_path = 0;
-
-  if (path) //Else EINVAL
-  {
-    if (resolved_path)
-    {
-      return_path = resolved_path;
-    }
-    else
-    {
-      //Non standard extension that glibc uses
-      return_path = (char*)malloc(PATH_MAX);
-    }
-
-    if (return_path) //Else EINVAL
-    {
-      //This is a Win32 API function similar to what realpath() is supposed to do
-      size_t size = GetFullPathNameA(path, PATH_MAX, return_path, 0);
-
-      //GetFullPathNameA() returns a size larger than buffer if buffer is too small
-      if (size > PATH_MAX)
-      {
-        if (return_path != resolved_path) //Malloc'd buffer - Unstandard extension retry
-        {
-          size_t new_size;
-
-          free(return_path);
-          return_path = (char*)malloc(size);
-
-          if (return_path)
-          {
-            new_size = GetFullPathNameA(path, size, return_path, 0); //Try again
-
-            if (new_size > size) //If it's still too large, we have a problem, don't try again
-            {
-              free(return_path);
-              return_path = 0;
-              errno = ENAMETOOLONG;
-            }
-            else
-            {
-              size = new_size;
-            }
-          }
-          else
-          {
-            //I wasn't sure what to return here, but the standard does say to return EINVAL
-            //if resolved_path is null, and in this case we couldn't malloc large enough buffer
-            errno = EINVAL;
-          }
-        }
-        else //resolved_path buffer isn't big enough
-        {
-          return_path = 0;
-          errno = ENAMETOOLONG;
-        }
-      }
-
-      //GetFullPathNameA() returns 0 if some path resolve problem occured
-      if (!size)
-      {
-        if (return_path != resolved_path) //Malloc'd buffer
-        {
-          free(return_path);
-        }
-
-        return_path = 0;
-
-        //Convert MS errors into standard errors
-        switch (GetLastError())
-        {
-          case ERROR_FILE_NOT_FOUND:
-            errno = ENOENT;
-            break;
-
-          case ERROR_PATH_NOT_FOUND: case ERROR_INVALID_DRIVE:
-            errno = ENOTDIR;
-            break;
-
-          case ERROR_ACCESS_DENIED:
-            errno = EACCES;
-            break;
-
-          default: //Unknown Error
-            errno = EIO;
-            break;
-        }
-      }
-
-      //If we get to here with a valid return_path, we're still doing good
-      if (return_path)
-      {
-        struct stat stat_buffer;
-
-        //Make sure path exists, stat() returns 0 on success
-        if (stat(return_path, &stat_buffer))
-        {
-          if (return_path != resolved_path)
-          {
-            free(return_path);
-          }
-
-          return_path = 0;
-          //stat() will set the correct errno for us
-        }
-        //else we succeeded!
-      }
-    }
-    else
-    {
-      errno = EINVAL;
-    }
-  }
-  else
-  {
-    errno = EINVAL;
-  }
-
-  if (return_path == NULL)
-  {
-    const char* fmt = "System.realpath failed on %s with errno: %d";
-    char* msg = (char*)malloc(strlen(path) + strlen(fmt) + 10);
-    sprintf(msg, fmt, path, errno);
-    c_add_message(NULL,6000,
-      ErrorType_scripting,
-      ErrorLevel_warning,
-      msg,
-      NULL,
-      0);
-    resolved_path = (char*)path;
-    return_path = (char*)path;
-  }
-
-  return return_path;
-}
-#endif /* mingw */
-
-#endif /* mingw and msvc */
-
 int System_getTerminalWidth(void)
 {
 #if defined(__MINGW32__) || defined(_MSC_VER)
@@ -2643,9 +2598,24 @@ int System_getTerminalWidth(void)
 
 #include "simulation_options.h"
 
+#define SB_SIZE 8192*4
+#define SB_SIZE_MINUS_ONE (SB_SIZE-1)
+
+/* snprintf check negative size */
+static size_t check_nonnegative(long sz)
+{
+  if (sz < 0) {
+    fprintf(stderr, "%s:%d got negative size: %ld, which should not happen\n", __FILE__, __LINE__, sz);
+    exit(EXIT_FAILURE);
+  }
+  return sz;
+}
+
+#define CHECK_NONNEGATIVE_BUFFER() check_nonnegative((long)SB_SIZE_MINUS_ONE-(cur-buf))
+
 char* System_getSimulationHelpTextSphinx(int detailed, int sphinx)
 {
-  static char buf[8192];
+  static char buf[SB_SIZE];
   int i,j;
   const char **desc = detailed ? FLAG_DETAILED_DESC : FLAG_DESC;
   char *cur = buf;
@@ -2653,13 +2623,13 @@ char* System_getSimulationHelpTextSphinx(int detailed, int sphinx)
   for(i=1; i<FLAG_MAX; ++i)
   {
     if (sphinx) {
-      cur += snprintf(cur, 8191-(buf-cur), "\n.. _simflag-%s :\n\n", FLAG_NAME[i]);
+      cur += snprintf(cur, CHECK_NONNEGATIVE_BUFFER(), "\n.. _simflag-%s :\n\n", FLAG_NAME[i]);
     }
     if (FLAG_TYPE[i] == FLAG_TYPE_FLAG) {
       if (sphinx) {
-        cur += snprintf(cur, 8191-(buf-cur), ":ref:`-%s <simflag-%s>`\n%s\n", FLAG_NAME[i], FLAG_NAME[i], desc[i]);
+        cur += snprintf(cur, CHECK_NONNEGATIVE_BUFFER(), ":ref:`-%s <simflag-%s>`\n%s\n", FLAG_NAME[i], FLAG_NAME[i], desc[i]);
       } else {
-        cur += snprintf(cur, 8191-(buf-cur), "<-%s>\n%s\n", FLAG_NAME[i], desc[i]);
+        cur += snprintf(cur, CHECK_NONNEGATIVE_BUFFER(), "<-%s>\n%s\n", FLAG_NAME[i], desc[i]);
       }
     } else if (FLAG_TYPE[i] == FLAG_TYPE_OPTION) {
       int numExtraFlags=0;
@@ -2667,28 +2637,41 @@ char* System_getSimulationHelpTextSphinx(int detailed, int sphinx)
       const char **flagName;
       const char **flagDesc;
       if (sphinx) {
-        cur += snprintf(cur, 8191-(buf-cur), ":ref:`-%s=value <simflag-%s>` *or* -%s value \n%s\n", FLAG_NAME[i], FLAG_NAME[i], FLAG_NAME[i], desc[i]);
+        cur += snprintf(cur, CHECK_NONNEGATIVE_BUFFER(), ":ref:`-%s=value <simflag-%s>` *or* -%s value \n%s\n", FLAG_NAME[i], FLAG_NAME[i], FLAG_NAME[i], desc[i]);
       } else {
-        cur += snprintf(cur, 8191-(buf-cur), "<-%s=value> or <-%s value>\n%s\n", FLAG_NAME[i], FLAG_NAME[i], desc[i]);
+        cur += snprintf(cur, CHECK_NONNEGATIVE_BUFFER(), "<-%s=value> or <-%s value>\n%s\n", FLAG_NAME[i], FLAG_NAME[i], desc[i]);
       }
 
       switch(i) {
+
+      case FLAG_IDA_LS:
+        numExtraFlags = IDA_LS_MAX;
+        flagName = IDA_LS_METHOD;
+        flagDesc = IDA_LS_METHOD_DESC;
+        break;
+
+      case FLAG_IIM:
+        numExtraFlags = IIM_MAX;
+        flagName = INIT_METHOD_NAME;
+        flagDesc = INIT_METHOD_DESC;
+        break;
+
+      case FLAG_JACOBIAN:
+        numExtraFlags = JAC_MAX;
+        flagName = JACOBIAN_METHOD;
+        flagDesc = JACOBIAN_METHOD_DESC;
+        break;
+
       case FLAG_LS:
         numExtraFlags = LS_MAX;
         flagName = LS_NAME;
         flagDesc = LS_DESC;
         break;
 
-      case FLAG_NLS:
-        numExtraFlags = NLS_MAX;
-        flagName = NLS_NAME;
-        flagDesc = NLS_DESC;
-        break;
-
-      case FLAG_NEWTON_STRATEGY:
-        numExtraFlags = NEWTON_MAX;
-        flagName = NEWTONSTRATEGY_NAME;
-        flagDesc = NEWTONSTRATEGY_DESC;
+      case FLAG_LSS:
+        numExtraFlags = LSS_MAX;
+        flagName = LSS_NAME;
+        flagDesc = LSS_DESC;
         break;
 
       case FLAG_LV:
@@ -2698,12 +2681,25 @@ char* System_getSimulationHelpTextSphinx(int detailed, int sphinx)
         flagDesc = LOG_STREAM_DESC;
         break;
 
-      case FLAG_IIM:
-        numExtraFlags = IIM_MAX;
-        flagName = INIT_METHOD_NAME;
-        flagDesc = INIT_METHOD_DESC;
+      case FLAG_NEWTON_STRATEGY:
+        numExtraFlags = NEWTON_MAX;
+        flagName = NEWTONSTRATEGY_NAME;
+        flagDesc = NEWTONSTRATEGY_DESC;
         break;
 
+      case FLAG_NLS:
+        numExtraFlags = NLS_MAX;
+        flagName = NLS_NAME;
+        flagDesc = NLS_DESC;
+        break;
+
+      case FLAG_NLS_LS:
+        numExtraFlags = NLS_LS_MAX;
+        flagName = NLS_LS_METHOD;
+        flagDesc = NLS_LS_METHOD_DESC;
+        break;
+
+
       case FLAG_S:
         numExtraFlags = S_MAX;
         flagName = NULL;
@@ -2712,20 +2708,20 @@ char* System_getSimulationHelpTextSphinx(int detailed, int sphinx)
       }
 
       if (numExtraFlags) {
-        cur += snprintf(cur, 8191-(buf-cur), "\n");
+        cur += snprintf(cur, CHECK_NONNEGATIVE_BUFFER(), "\n");
         if (flagName) {
           for (j=firstExtraFlag; j<numExtraFlags; j++) {
-            cur += snprintf(cur, 8191-(buf-cur), "  * %s (%s)\n", flagName[j], flagDesc[j]);
+            cur += snprintf(cur, CHECK_NONNEGATIVE_BUFFER(), "  * %s (%s)\n", flagName[j], flagDesc[j]);
           }
         } else {
           for (j=firstExtraFlag; j<numExtraFlags; j++) {
-            cur += snprintf(cur, 8191-(buf-cur), "  * %s\n", flagDesc[j]);
+            cur += snprintf(cur, CHECK_NONNEGATIVE_BUFFER(), "  * %s\n", flagDesc[j]);
           }
         }
       }
 
     } else {
-      cur += snprintf(cur, 8191-(buf-cur), "[unknown flag-type] <-%s>\n", FLAG_NAME[i]);
+      cur += snprintf(cur, CHECK_NONNEGATIVE_BUFFER(), "[unknown flag-type] <-%s>\n", FLAG_NAME[i]);
     }
   }
   *cur = 0;
@@ -2790,7 +2786,7 @@ int SystemImpl__fileIsNewerThan(const char *file1, const char *file2)
 
 void SystemImpl__initGarbageCollector(void)
 {
-  static init=0;
+  static int init=0;
   if (!init) {
     GC_init();
     GC_register_displacement(0);
@@ -3087,11 +3083,6 @@ const char* SystemImpl__createTemporaryDirectory(const char *templatePrefix)
   MMC_THROW();
 }
 
-const char* System_getTriple()
-{
-  return DEFAULT_TRIPLE;
-}
-
 #if defined(OMC_GENERATE_RELOCATABLE_CODE)
 static void addDLError(const char *msg, const char *fileName)
 {
