@@ -39,6 +39,7 @@
 #include "../../simulation/results/simulation_result.h"
 #include "../../simulation/options.h"
 #include "../../simulation/solver/model_help.h"
+#include "../../util/omc_file.h"
 
 static inline void pickUpDim(OptDataDim * dim, DATA* data, OptDataTime * time);
 static inline void pickUpTime(OptDataTime * time, OptDataDim * dim, DATA* data, const double preSimTime);
@@ -248,7 +249,7 @@ static int getNsi(char*filename, const int nsi, modelica_boolean * exTimeGrid){
   FILE * pFile = NULL;
 
   *exTimeGrid = 0;
-  pFile = fopen(filename,"r");
+  pFile = omc_fopen(filename,"r");
   if(pFile == NULL){
     warningStreamPrint(LOG_STDOUT, 0, "OMC can't find the file %s.", filename);
     fclose(pFile);
@@ -275,7 +276,7 @@ static inline void overwriteTimeGridFile(OptDataTime * time, char* filename, lon
   const int np1 = np - 1;
   double t;
   FILE * pFile = NULL;
-  pFile = fopen(filename,"r");
+  pFile = omc_fopen(filename,"r");
 
   fscanf(pFile, "%lf", &t);
   time->t0 = t;
@@ -817,15 +818,16 @@ void diffSynColoredOptimizerSystem(OptData *optData, modelica_real **J, const in
   int i,j,l,ii, ll;
 
   const int h_index = optData->s.indexABCD[index];
+  ANALYTIC_JACOBIAN* jacobian = &(data->simulationInfo->analyticJacobians[h_index]);
   const long double * scaldt = optData->bounds.scaldt[m];
-  const unsigned int * const cC = data->simulationInfo->analyticJacobians[h_index].sparsePattern.colorCols;
-  const unsigned int * const lindex  = data->simulationInfo->analyticJacobians[h_index].sparsePattern.leadindex;
-  const int nx = data->simulationInfo->analyticJacobians[h_index].sizeCols;
-  const int Cmax = data->simulationInfo->analyticJacobians[h_index].sparsePattern.maxColors + 1;
+  const unsigned int * const cC = jacobian->sparsePattern->colorCols;
+  const unsigned int * const lindex  = jacobian->sparsePattern->leadindex;
+  const int nx = jacobian->sizeCols;
+  const int Cmax = jacobian->sparsePattern->maxColors + 1;
   const int dnx = optData->dim.nx;
   const int dnxnc = optData->dim.nJ;
-  const modelica_real * const resultVars = data->simulationInfo->analyticJacobians[h_index].resultVars;
-  const unsigned int * const sPindex = data->simulationInfo->analyticJacobians[h_index].sparsePattern.index;
+  const modelica_real * const resultVars = jacobian->resultVars;
+  const unsigned int * const sPindex = jacobian->sparsePattern->index;
   long double  scalb = optData->bounds.scalb[m][n];
 
   const int * index_J = (index == 3)? optData->s.indexJ3 : optData->s.indexJ2;
@@ -833,16 +835,21 @@ void diffSynColoredOptimizerSystem(OptData *optData, modelica_real **J, const in
 
   modelica_real **sV = optData->s.seedVec[index];
 
+  /* set symbolic jacobian context to reuse the matrix and the factorization in every column */
+  setContext(data, &(data->localData[0]->timeValue), CONTEXT_SYM_JACOBIAN);
+
   for(i = 1; i < Cmax; ++i){
-    data->simulationInfo->analyticJacobians[h_index].seedVars = sV[i];
+    jacobian->seedVars = sV[i];
 
     if(index == 2){
-      data->callback->functionJacB_column(data, threadData);
+      data->callback->functionJacB_column(data, threadData, jacobian, NULL);
     }else if(index == 3){
-      data->callback->functionJacC_column(data, threadData);
+      data->callback->functionJacC_column(data, threadData, jacobian, NULL);
     }else
       assert(0);
 
+    increaseJacContext(data);
+
     for(ii = 0; ii < nx; ++ii){
       if(cC[ii] == i){
         for(j = lindex[ii]; j < lindex[ii + 1]; ++j){
@@ -862,6 +869,8 @@ void diffSynColoredOptimizerSystem(OptData *optData, modelica_real **J, const in
 
     }
   }
+  /* set context for the start values extrapolation of non-linear algebraic loops */
+  unsetContext(data);
 }
 
 void diffSynColoredOptimizerSystemF(OptData *optData, modelica_real **J){
@@ -871,19 +880,25 @@ void diffSynColoredOptimizerSystemF(OptData *optData, modelica_real **J){
     int i,j,l,ii, ll;
     const int index = 4;
     const int h_index = optData->s.indexABCD[index];
-    const unsigned int * const cC = data->simulationInfo->analyticJacobians[h_index].sparsePattern.colorCols;
-    const unsigned int * const lindex  = data->simulationInfo->analyticJacobians[h_index].sparsePattern.leadindex;
-    const int nx = data->simulationInfo->analyticJacobians[h_index].sizeCols;
-    const int Cmax = data->simulationInfo->analyticJacobians[h_index].sparsePattern.maxColors + 1;
-    const modelica_real * const resultVars = data->simulationInfo->analyticJacobians[h_index].resultVars;
-    const unsigned int * const sPindex = data->simulationInfo->analyticJacobians[h_index].sparsePattern.index;
+    ANALYTIC_JACOBIAN* jacobian = &(data->simulationInfo->analyticJacobians[h_index]);
+    const unsigned int * const cC = jacobian->sparsePattern->colorCols;
+    const unsigned int * const lindex  = jacobian->sparsePattern->leadindex;
+    const int nx = jacobian->sizeCols;
+    const int Cmax = jacobian->sparsePattern->maxColors + 1;
+    const modelica_real * const resultVars = jacobian->resultVars;
+    const unsigned int * const sPindex = jacobian->sparsePattern->index;
 
     modelica_real **sV = optData->s.seedVec[index];
 
-    for(i = 1; i < Cmax; ++i){
-      data->simulationInfo->analyticJacobians[h_index].seedVars = sV[i];
+    /* set symbolic jacobian context to reuse the matrix and the factorization in every column */
+    setContext(data,  &(data->localData[0]->timeValue), CONTEXT_SYM_JACOBIAN);
 
-      data->callback->functionJacD_column(data, threadData);
+    for(i = 1; i < Cmax; ++i){
+      jacobian->seedVars = sV[i];
+
+      data->callback->functionJacD_column(data, threadData, jacobian, NULL);
+
+      increaseJacContext(data);
 
       for(ii = 0; ii < nx; ++ii){
         if(cC[ii] == i){
@@ -894,6 +909,8 @@ void diffSynColoredOptimizerSystemF(OptData *optData, modelica_real **J){
         }
       }
     }
+    /* set context for the start values extrapolation of non-linear algebraic loops */
+    unsetContext(data);
   }
 }
 
@@ -907,7 +924,8 @@ static inline void pickUpStates(OptData* optData){
 
   if(cflags){
     FILE * pFile = NULL;
-    pFile = fopen(cflags,"r");
+    pFile = omc_fopen(cflags,"r");
+
     if(pFile == NULL){
       warningStreamPrint(LOG_STDOUT, 0, "OMC can't find the file %s.",cflags);
     }else{
