@@ -46,7 +46,7 @@
 #include "simulation/solver/external_input.h"
 #include "simulation/solver/epsilon.h"
 #include "simulation/solver/omc_math.h"
-
+#include "simulation/solver/jacobianSymbolical.h"
 #include "simulation/solver/dassl.h"
 #include "meta/meta_modelica.h"
 
@@ -62,27 +62,49 @@ extern "C" {
 int RHSFinalFlag;
 
 /* provides a dummy Jacobian to be used with DASSL */
-static int
-dummy_Jacobian(double *t, double *y, double *yprime, double *deltaD,
-    double *delta, double *cj, double *h, double *wt, double *rpar, int* ipar) {
-  return 0;
-}
-static int
-dummy_zeroCrossing(int *neqm, double *t, double *y, double *yp,
-                   int *ng, double *gout, double *rpar, int* ipar) {
+static int dummy_Jacobian(double *t, double *y, double *yprime,double *deltaD,
+                          double *delta, double *cj, double *h, double *wt,
+                          double *rpar, int* ipar) {
   return 0;
 }
 
-static int callJacobian(double *t, double *y, double *yprime, double *deltaD, double *pd, double *cj, double *h, double *wt,
-   double *rpar, int* ipar);
-static int jacA_num(double *t, double *y, double *yprime, double *deltaD, double *pd, double *cj, double *h, double *wt,
-   double *rpar, int* ipar);
-static int jacA_numColored(double *t, double *y, double *yprime, double *deltaD, double *pd, double *cj, double *h, double *wt,
-   double *rpar, int* ipar);
-static int jacA_sym(double *t, double *y, double *yprime, double *deltaD, double *pd, double *cj, double *h, double *wt,
-       double *rpar, int* ipar);
-static int jacA_symColored(double *t, double *y, double *yprime, double *deltaD, double *pd, double *cj, double *h, double *wt,
-       double *rpar, int* ipar);
+/* provides a dummy zero crossing function to be used with DASSL */
+static int dummy_zeroCrossing(int *neqm, double *t, double *y, double *yp,
+                              int *ng, double *gout, double *rpar, int* ipar) {
+  return 0;
+}
+
+/* provides a dumm precondition function to be used with DASSL */
+static int dummy_precondition(int *neq, double *t, double *y, double *yprime,
+                              double *savr, double *pwk, double *cj,
+                              double *wt, double *wp, int *iwp, double *b,
+                              double eplin, int* ires, double *rpar, int* ipar){
+    return 0;
+}
+
+/* Function prototypes */
+static int callJacobian(double *t, double *y, double *yprime, double *deltaD,
+                        double *pd, double *cj, double *h, double *wt,
+                        double *rpar, int* ipar);
+
+static int jacA_num(double *t, double *y, double *yprime, double *deltaD,
+                    double *pd, double *cj, double *h, double *wt,
+                    double *rpar, int* ipar);
+
+static int jacA_numColored(double *t, double *y, double *yprime,
+                           double *deltaD, double *pd, double *cj, double *h,
+                           double *wt, double *rpar, int* ipar);
+
+static int jacA_sym(double *t, double *y, double *yprime, double *deltaD,
+                    double *pd, double *cj, double *h, double *wt,
+                    double *rpar, int* ipar);
+
+static int jacA_symColored(double *t, double *y, double *yprime,
+                           double *deltaD, double *pd, double *cj, double *h,
+                           double *wt, double *rpar, int* ipar);
+
+static void setJacElementDasslSparse(int l, int k, int nth, double val,
+                                     void* matrixA, int rows);
 
 void  DDASKR(
     int (*res) (double *t, double *y, double *yprime, double* cj, double *delta, int *ires, double *rpar, int* ipar),
@@ -108,20 +130,35 @@ void  DDASKR(
     int *jroot
 );
 
-static int
-dummy_precondition(int *neq, double *t, double *y, double *yprime, double *savr, double *pwk, double *cj, double *wt, double *wp, int *iwp, double *b, double eplin, int* ires, double *rpar, int* ipar){
-    return 0;
-}
-
 static int continue_DASSL(int* idid, double* tolarence);
 
 /* function for calculating state values on residual form */
-static int functionODE_residual(double *t, double *x, double *xprime, double *cj, double *delta, int *ires, double *rpar, int* ipar);
-/* function for calculating zeroCrossings */
-static int function_ZeroCrossingsDASSL(int *neqm, double *t, double *y, double *yp,
-        int *ng, double *gout, double *rpar, int* ipar);
+static int functionODE_residual(double *t, double *x, double *xprime,
+                                double *cj, double *delta, int *ires,
+                                double *rpar, int* ipar);
 
-int dassl_initial(DATA* data, threadData_t *threadData, SOLVER_INFO* solverInfo, DASSL_DATA *dasslData)
+/* function for calculating zeroCrossings */
+static int function_ZeroCrossingsDASSL(int *neqm, double *t, double *y,
+                                       double *yp, int *ng, double *gout,
+                                       double *rpar, int* ipar);
+
+
+/*
+ * \brief Configure DASSL solver
+ *
+ * Allocate memory for intern data of `dasslData`.
+ * Configures DASSL:
+ *   - Set relative and absolute tolerance
+ *   - Set maximum step size, initial step size
+ *   - Set maximum integration order
+ *   - Set time grid
+ *   - Set method for jacobian computation
+ *   - Set root finding method
+ *   - Set event handling and restart option
+ *
+ */
+int dassl_initial(DATA* data, threadData_t *threadData,
+                  SOLVER_INFO* solverInfo, DASSL_DATA *dasslData)
 {
   TRACE_PUSH
   /* work arrays for DASSL */
@@ -315,7 +352,7 @@ int dassl_initial(DATA* data, threadData_t *threadData, SOLVER_INFO* solverInfo,
       ANALYTIC_JACOBIAN* jac = &data->simulationInfo->analyticJacobians[data->callback->INDEX_JAC_A];
       infoStreamPrint(LOG_SIMULATION, 1, "Initialized colored Jacobian:");
       infoStreamPrint(LOG_SIMULATION, 0, "columns: %d rows: %d", jac->sizeCols, jac->sizeRows);
-      infoStreamPrint(LOG_SIMULATION, 0, "NNZ:  %d colors: %d", jac->sparsePattern.numberOfNoneZeros, jac->sparsePattern.maxColors);
+      infoStreamPrint(LOG_SIMULATION, 0, "NNZ:  %d colors: %d", jac->sparsePattern->numberOfNoneZeros, jac->sparsePattern->maxColors);
       messageClose(LOG_SIMULATION);
     }
   }
@@ -325,11 +362,11 @@ int dassl_initial(DATA* data, threadData_t *threadData, SOLVER_INFO* solverInfo,
   /* set up the appropriate function pointer */
   switch (dasslData->dasslJacobian){
     case COLOREDNUMJAC:
-      data->simulationInfo->jacobianEvals = data->simulationInfo->analyticJacobians[data->callback->INDEX_JAC_A].sparsePattern.maxColors;
+      data->simulationInfo->jacobianEvals = data->simulationInfo->analyticJacobians[data->callback->INDEX_JAC_A].sparsePattern->maxColors;
       dasslData->jacobianFunction =  jacA_numColored;
       break;
     case COLOREDSYMJAC:
-      data->simulationInfo->jacobianEvals = data->simulationInfo->analyticJacobians[data->callback->INDEX_JAC_A].sparsePattern.maxColors;
+      data->simulationInfo->jacobianEvals = data->simulationInfo->analyticJacobians[data->callback->INDEX_JAC_A].sparsePattern->maxColors;
       dasslData->jacobianFunction =  jacA_symColored;
       break;
     case SYMJAC:
@@ -378,13 +415,15 @@ int dassl_initial(DATA* data, threadData_t *threadData, SOLVER_INFO* solverInfo,
 
   /* ### end configuration of dassl ### */
 
-
   messageClose(LOG_SOLVER);
   TRACE_POP
   return 0;
 }
 
 
+/*
+ * \brief Deallocates `DASSL_DATA`
+ */
 int dassl_deinitial(DASSL_DATA *dasslData)
 {
   TRACE_PUSH
@@ -486,6 +525,8 @@ int dassl_step(DATA* data, threadData_t *threadData, SOLVER_INFO* solverInfo)
 
   MODEL_DATA *mData = (MODEL_DATA*) data->modelData;
 
+  if (measure_time_flag) rt_tick(SIM_TIMER_SOLVER);
+
   memcpy(stateDer, data->localData[1]->realVars + data->modelData->nStates, sizeof(double)*data->modelData->nStates);
 
   dasslData->rpar[0] = (double*) (void*) data;
@@ -557,9 +598,11 @@ int dassl_step(DATA* data, threadData_t *threadData, SOLVER_INFO* solverInfo)
     /* rhs final flag is FALSE during for dassl evaluation */
     RHSFinalFlag = 0;
 
+    if (measure_time_flag) rt_accumulate(SIM_TIMER_SOLVER);
     /* read input vars */
     externalInputUpdate(data);
     data->callback->input_function(data, threadData);
+    if (measure_time_flag) rt_tick(SIM_TIMER_SOLVER);
 
     DDASKR(dasslData->residualFunction, (int*) &dasslData->N,
             &solverInfo->currentTime, states, stateDer, &tout,
@@ -664,13 +707,13 @@ int dassl_step(DATA* data, threadData_t *threadData, SOLVER_INFO* solverInfo)
   }
 
   infoStreamPrint(LOG_DASSL, 0, "Finished DASSL step.");
+  if (measure_time_flag) rt_accumulate(SIM_TIMER_SOLVER);
 
   TRACE_POP
   return retVal;
 }
 
-static int
-continue_DASSL(int* idid, double* atol)
+static int continue_DASSL(int* idid, double* atol)
 {
   TRACE_PUSH
   int retValue = -1;
@@ -733,9 +776,8 @@ continue_DASSL(int* idid, double* atol)
   return retValue;
 }
 
-
-int functionODE_residual(double *t, double *y, double *yd, double* cj, double *delta,
-                    int *ires, double *rpar, int *ipar)
+int functionODE_residual(double *t, double *y, double *yd, double* cj,
+                         double *delta, int *ires, double *rpar, int *ipar)
 {
   TRACE_PUSH
   DATA* data = (DATA*)((double**)rpar)[0];
@@ -747,6 +789,9 @@ int functionODE_residual(double *t, double *y, double *yd, double* cj, double *d
   int saveJumpState;
   int success = 0;
 
+  if (measure_time_flag) rt_accumulate(SIM_TIMER_SOLVER);
+  if (measure_time_flag) rt_tick(SIM_TIMER_RESIDUALS);
+
   if (data->simulationInfo->currentContext == CONTEXT_ALGEBRAIC)
   {
     setContext(data, t, CONTEXT_ODE);
@@ -796,12 +841,15 @@ int functionODE_residual(double *t, double *y, double *yd, double* cj, double *d
     unsetContext(data);
   }
 
+  if (measure_time_flag) rt_accumulate(SIM_TIMER_RESIDUALS);
+  if (measure_time_flag) rt_tick(SIM_TIMER_SOLVER);
+
   TRACE_POP
   return 0;
 }
 
 int function_ZeroCrossingsDASSL(int *neqm, double *t, double *y, double *yp,
-        int *ng, double *gout, double *rpar, int* ipar)
+                                int *ng, double *gout, double *rpar, int* ipar)
 {
   TRACE_PUSH
   DATA* data = (DATA*)(void*)((double**)rpar)[0];
@@ -811,6 +859,9 @@ int function_ZeroCrossingsDASSL(int *neqm, double *t, double *y, double *yp,
   double timeBackup;
   int saveJumpState;
 
+  if (measure_time_flag) rt_accumulate(SIM_TIMER_SOLVER);
+  if (measure_time_flag) rt_tick(SIM_TIMER_EVENT);
+
   if (data->simulationInfo->currentContext == CONTEXT_ALGEBRAIC)
   {
     setContext(data, t, CONTEXT_EVENTS);
@@ -837,60 +888,47 @@ int function_ZeroCrossingsDASSL(int *neqm, double *t, double *y, double *yp,
     unsetContext(data);
   }
 
+  if (measure_time_flag) rt_accumulate(SIM_TIMER_EVENT);
+  if (measure_time_flag) rt_tick(SIM_TIMER_SOLVER);
+
   TRACE_POP
   return 0;
 }
 
+/*
+ * Sets element (l,j) in matrixA to given value val.
+ */
+void setJacElementDasslSparse(int l, int j, int nth, double val, void* matrixA,
+                              int rows)
+{
+  int k  = j*rows + l;
+  ((double*) matrixA)[k]=val;
+}
+
 /* \fn jacA_symColored(double *t, double *y, double *yprime, double *deltaD, double *pd, double *cj, double *h, double *wt,
    double *rpar, int* ipar)
  *
  *
  * This function calculates symbolically the jacobian matrix and exploiting the coloring.
  */
-int jacA_symColored(double *t, double *y, double *yprime, double *delta, double *matrixA, double *cj, double *h, double *wt, double *rpar, int *ipar)
+int jacA_symColored(double *t, double *y, double *yprime, double *delta,
+                    double *matrixA, double *cj, double *h, double *wt,
+                    double *rpar, int *ipar)
 {
   TRACE_PUSH
   DATA* data = (DATA*)(void*)((double**)rpar)[0];
-  DASSL_DATA* dasslData = (DASSL_DATA*)(void*)((double**)rpar)[1];
   threadData_t *threadData = (threadData_t*)(void*)((double**)rpar)[2];
+  DASSL_DATA* dasslData = (DASSL_DATA*)(void*)((double**)rpar)[1];
 
   const int index = data->callback->INDEX_JAC_A;
-  ANALYTIC_JACOBIAN* jacobian = &(data->simulationInfo->analyticJacobians[index]);
+  ANALYTIC_JACOBIAN* jac = &(data->simulationInfo->analyticJacobians[index]);
+  unsigned int columns = jac->sizeCols;
+  unsigned int rows = jac->sizeRows;
+  unsigned int sizeTmpVars = jac->sizeTmpVars;
+  SPARSE_PATTERN* spp = jac->sparsePattern;
 
-  unsigned int i,j,l,k,ii;
-
-  /* set symbolical jacobian to reuse the matrix A and the factorization
-   * in the Linear loops of  functionJacA_column */
-  setContext(data, t, CONTEXT_SYM_JACOBIAN);
-
-  for(i=0; i < jacobian->sparsePattern.maxColors; i++)
-  {
-    for(ii=0; ii < jacobian->sizeCols; ii++)
-      if(jacobian->sparsePattern.colorCols[ii]-1 == i)
-        jacobian->seedVars[ii] = 1;
-
-    data->callback->functionJacA_column(data, threadData, jacobian, NULL);
-
-    increaseJacContext(data);
-
-    for(j = 0; j < jacobian->sizeCols; j++)
-    {
-      if(jacobian->seedVars[j] == 1)
-      {
-        ii = jacobian->sparsePattern.leadindex[j];
-        while(ii < jacobian->sparsePattern.leadindex[j+1])
-        {
-          l  = jacobian->sparsePattern.index[ii];
-          k  = j*jacobian->sizeRows + l;
-          matrixA[k] = jacobian->resultVars[l];
-          ii++;
-        };
-      }
-    }
-    for(ii=0; ii < jacobian->sizeCols; ii++)
-      if(jacobian->sparsePattern.colorCols[ii]-1 == i) jacobian->seedVars[ii] = 0;
-
-  }
+  genericColoredSymbolicJacobianEvaluation(rows, columns, spp, matrixA, jac,
+                                           data, threadData, &setJacElementDasslSparse);
 
   TRACE_POP
   return 0;
@@ -902,37 +940,33 @@ int jacA_symColored(double *t, double *y, double *yprime, double *delta, double
  *
  * This function calculates symbolically the jacobian matrix.
  */
-int jacA_sym(double *t, double *y, double *yprime, double *delta, double *matrixA, double *cj, double *h, double *wt, double *rpar, int *ipar)
+int jacA_sym(double *t, double *y, double *yprime, double *delta,
+             double *matrixA, double *cj, double *h, double *wt, double *rpar,
+             int *ipar)
 {
   TRACE_PUSH
+
   DATA* data = (DATA*)(void*)((double**)rpar)[0];
   DASSL_DATA* dasslData = (DASSL_DATA*)(void*)((double**)rpar)[1];
   threadData_t *threadData = (threadData_t*)(void*)((double**)rpar)[2];
 
   const int index = data->callback->INDEX_JAC_A;
-  ANALYTIC_JACOBIAN* jacobian = &(data->simulationInfo->analyticJacobians[index]);
+  ANALYTIC_JACOBIAN* t_jac = &(data->simulationInfo->analyticJacobians[index]);
+  unsigned int columns = t_jac->sizeCols;
+  unsigned int rows = t_jac->sizeRows;
+  unsigned int sizeTmpVars = t_jac->sizeTmpVars;
 
-  unsigned int i,j,k;
+  unsigned int i,j;
 
-  /* set symbolical jacobian to reuse the matrix A and the factorization
-   * in the Linear loops of  functionJacA_column */
-  setContext(data, t, CONTEXT_SYM_JACOBIAN);
-
-  k = 0;
-  for(i=0; i < jacobian->sizeCols; i++)
+  for(i=0; i < columns; i++)
   {
-	 jacobian->seedVars[i] = 1.0;
+    t_jac->seedVars[i] = 1.0;
+    data->callback->functionJacA_column(data, threadData, t_jac, NULL);
 
-    data->callback->functionJacA_column(data, threadData, jacobian, NULL);
+    for(j = 0; j < rows; j++)
+      matrixA[i*columns+j] = t_jac->resultVars[j];
 
-    increaseJacContext(data);
-
-    for(j = 0; j < jacobian->sizeRows; j++)
-    {
-      matrixA[i*jacobian->sizeCols+j] = jacobian->resultVars[j];
-    }
-
-    jacobian->seedVars[i] = 0.0;
+    t_jac->seedVars[i] = 0.0;
   }
 
   TRACE_POP
@@ -946,7 +980,9 @@ int jacA_sym(double *t, double *y, double *yprime, double *delta, double *matrix
  * This function calculates a jacobian matrix by
  * numerical with forward finite differences.
  */
-int jacA_num(double *t, double *y, double *yprime, double *delta, double *matrixA, double *cj, double *h, double *wt, double *rpar, int *ipar)
+int jacA_num(double *t, double *y, double *yprime, double *delta,
+             double *matrixA, double *cj, double *h, double *wt, double *rpar,
+             int *ipar)
 {
   TRACE_PUSH
   DATA* data = (DATA*)(void*)((double**)rpar)[0];
@@ -1001,7 +1037,9 @@ int jacA_num(double *t, double *y, double *yprime, double *delta, double *matrix
  * This function calculates a jacobian matrix by
  * numerical with forward finite differences and exploiting the coloring.
  */
-int jacA_numColored(double *t, double *y, double *yprime, double *delta, double *matrixA, double *cj, double *h, double *wt, double *rpar, int *ipar)
+int jacA_numColored(double *t, double *y, double *yprime, double *delta,
+                    double *matrixA, double *cj, double *h, double *wt,
+                    double *rpar, int *ipar)
 {
   TRACE_PUSH
 
@@ -1024,11 +1062,11 @@ int jacA_numColored(double *t, double *y, double *yprime, double *delta, double
   /* set context for the start values extrapolation of non-linear algebraic loops */
   setContext(data, t, CONTEXT_JACOBIAN);
 
-  for(i = 0; i < jacobian->sparsePattern.maxColors; i++)
+  for(i = 0; i < jacobian->sparsePattern->maxColors; i++)
   {
     for(ii=0; ii < jacobian->sizeCols; ii++)
     {
-      if(jacobian->sparsePattern.colorCols[ii]-1 == i)
+      if(jacobian->sparsePattern->colorCols[ii]-1 == i)
       {
         delta_hhh = *h * yprime[ii];
         delta_hh[ii] = delta_h * fmax(fmax(fabs(y[ii]),fabs(delta_hhh)),fabs(1./wt[ii]));
@@ -1048,12 +1086,12 @@ int jacA_numColored(double *t, double *y, double *yprime, double *delta, double
 
     for(ii = 0; ii < jacobian->sizeCols; ii++)
     {
-      if(jacobian->sparsePattern.colorCols[ii]-1 == i)
+      if(jacobian->sparsePattern->colorCols[ii]-1 == i)
       {
-        j = jacobian->sparsePattern.leadindex[ii];
-        while(j < jacobian->sparsePattern.leadindex[ii+1])
+        j = jacobian->sparsePattern->leadindex[ii];
+        while(j < jacobian->sparsePattern->leadindex[ii+1])
         {
-          l  =  jacobian->sparsePattern.index[j];
+          l  =  jacobian->sparsePattern->index[j];
           k  = l + ii*jacobian->sizeRows;
           matrixA[k] = (dasslData->newdelta[l] - delta[l]) * delta_hh[ii];
           j++;
@@ -1074,8 +1112,9 @@ int jacA_numColored(double *t, double *y, double *yprime, double *delta, double
  * This function is called by dassl to calculate the jacobian matrix.
  *
  */
-static int callJacobian(double *t, double *y, double *yprime, double *deltaD, double *pd, double *cj, double *h, double *wt,
-   double *rpar, int* ipar)
+static int callJacobian(double *t, double *y, double *yprime, double *deltaD,
+                        double *pd, double *cj, double *h, double *wt,
+                        double *rpar, int* ipar)
 {
   TRACE_PUSH
   DATA* data = (DATA*)(void*)((double**)rpar)[0];
@@ -1083,6 +1122,7 @@ static int callJacobian(double *t, double *y, double *yprime, double *deltaD, do
   threadData_t *threadData = (threadData_t*)(void*)((double**)rpar)[2];
 
   /* profiling */
+  if (measure_time_flag) rt_accumulate(SIM_TIMER_SOLVER);
   rt_tick(SIM_TIMER_JACOBIAN);
 
   if(dasslData->jacobianFunction(t, y, yprime, deltaD, pd, cj, h, wt, rpar, ipar))
@@ -1092,9 +1132,6 @@ static int callJacobian(double *t, double *y, double *yprime, double *deltaD, do
     return 1;
   }
 
-  /* profiling */
-  rt_accumulate(SIM_TIMER_JACOBIAN);
-
   /* debug */
   if (ACTIVE_STREAM(LOG_JAC)){
     _omc_matrix* dumpJac = _omc_createMatrix(dasslData->N, dasslData->N, pd);
@@ -1111,6 +1148,10 @@ static int callJacobian(double *t, double *y, double *yprime, double *deltaD, do
   /* set context for the start values extrapolation of non-linear algebraic loops */
   unsetContext(data);
 
+  /* profiling */
+  rt_accumulate(SIM_TIMER_JACOBIAN);
+  if (measure_time_flag) rt_tick(SIM_TIMER_SOLVER);
+
   TRACE_POP
   return 0;
 }
