@@ -34,7 +34,9 @@
 #include <math.h>
 #include <string.h>
 
-#include "util/omc_error.h"
+#include "../../util/simulation_options.h"
+#include "../../util/omc_error.h"
+#include "../../util/omc_file.h"
 #include "nonlinearSystem.h"
 #include "nonlinearValuesList.h"
 #if !defined(OMC_MINIMAL_RUNTIME)
@@ -44,25 +46,32 @@
 #include "newtonIteration.h"
 #endif
 #include "nonlinearSolverHomotopy.h"
-#include "simulation/simulation_info_json.h"
-#include "simulation/simulation_runtime.h"
-#include "simulation/solver/model_help.h"
+#include "../options.h"
+#include "../simulation_info_json.h"
+#include "../simulation_runtime.h"
+#include "model_help.h"
 
 /* for try and catch simulationJumpBuffer */
-#include "meta/meta_modelica.h"
+#include "../../meta/meta_modelica.h"
 
 int check_nonlinear_solution(DATA *data, int printFailingSystems, int sysNumber);
 
 extern int init_lambda_steps;
 
-struct dataNewtonAndHybrid
+struct dataSolver
 {
-  void* newtonData;
+  void* ordinaryData;
+  void* initHomotopyData;
+};
+
+struct dataMixedSolver
+{
+  void* newtonHomotopyData;
   void* hybridData;
 };
 
 #if !defined(OMC_MINIMAL_RUNTIME)
-#include "util/write_csv.h"
+#include "../../util/write_csv.h"
 /*! \fn int initializeNLScsvData(DATA* data, NONLINEAR_SYSTEM_DATA* systemData)
  *
  *  This function initializes csv files for analysis propose.
@@ -343,10 +352,22 @@ int initializeNonlinearSystems(DATA *data, threadData_t *threadData)
   int i,j;
   int size, nnz;
   NONLINEAR_SYSTEM_DATA *nonlinsys = data->simulationInfo->nonlinearSystemData;
-  struct dataNewtonAndHybrid *mixedSolverData;
+  struct dataSolver *solverData;
+  struct dataMixedSolver *mixedSolverData;
 
   infoStreamPrint(LOG_NLS, 1, "initialize non-linear system solvers");
   infoStreamPrint(LOG_NLS, 0, "%ld non-linear systems", data->modelData->nNonLinearSystems);
+  if (data->simulationInfo->nlsLinearSolver == NLS_LS_DEFAULT) {
+#if !defined(OMC_MINIMAL_RUNTIME)
+    if (data->simulationInfo->nlsMethod == NLS_KINSOL) {
+      data->simulationInfo->nlsLinearSolver = NLS_LS_KLU;
+    } else {
+      data->simulationInfo->nlsLinearSolver = NLS_LS_LAPACK;
+    }
+#else
+    data->simulationInfo->nlsLinearSolver = NLS_LS_LAPACK;
+#endif
+  }
 
   for(i=0; i<data->modelData->nNonLinearSystems; ++i)
   {
@@ -360,8 +381,9 @@ int initializeNonlinearSystems(DATA *data, threadData_t *threadData)
     /* check if analytical jacobian is created */
     if(nonlinsys[i].jacobianIndex != -1)
     {
+      ANALYTIC_JACOBIAN* jacobian = &(data->simulationInfo->analyticJacobians[nonlinsys[i].jacobianIndex]);
       assertStreamPrint(threadData, 0 != nonlinsys[i].analyticalJacobianColumn, "jacobian function pointer is invalid" );
-      if(nonlinsys[i].initialAnalyticalJacobian(data, threadData))
+      if(nonlinsys[i].initialAnalyticalJacobian(data, threadData, jacobian))
       {
         nonlinsys[i].jacobianIndex = -1;
       }
@@ -370,7 +392,7 @@ int initializeNonlinearSystems(DATA *data, threadData_t *threadData)
 #if !defined(OMC_MINIMAL_RUNTIME)
     if (nonlinsys[i].isPatternAvailable)
     {
-      nnz = nonlinsys[i].sparsePattern.numberOfNoneZeros;
+      nnz = nonlinsys[i].sparsePattern->numberOfNoneZeros;
 
       if(nnz/(double)(size*size)<=nonlinearSparseSolverMaxDensity && size >= nonlinearSparseSolverMinSize)
       {
@@ -412,39 +434,67 @@ int initializeNonlinearSystems(DATA *data, threadData_t *threadData)
     }
 #endif
 
-    /* allocate solver data */
     switch(data->simulationInfo->nlsMethod)
     {
 #if !defined(OMC_MINIMAL_RUNTIME)
     case NLS_HYBRID:
-      allocateHybrdData(size, &nonlinsys[i].solverData);
+      solverData = (struct dataSolver*) malloc(sizeof(struct dataSolver));
+      if (nonlinsys[i].homotopySupport && (data->callback->useHomotopy == 2 || data->callback->useHomotopy == 3)) {
+        allocateHybrdData(size-1, &(solverData->ordinaryData));
+        allocateHomotopyData(size-1, &(solverData->initHomotopyData));
+      } else {
+        allocateHybrdData(size, &(solverData->ordinaryData));
+      }
+      nonlinsys[i].solverData = (void*) solverData;
       break;
     case NLS_KINSOL:
-      nlsKinsolAllocate(size, &nonlinsys[i], data->simulationInfo->nlsLinearSolver);
+      solverData = (struct dataSolver*) malloc(sizeof(struct dataSolver));
+      if (nonlinsys[i].homotopySupport && (data->callback->useHomotopy == 2 || data->callback->useHomotopy == 3)) {
+        // Try without homotopy not supported for kinsol
+        // nlsKinsolAllocate(size-1, &nonlinsys[i], data->simulationInfo->nlsLinearSolver);
+        // solverData->ordinaryData = nonlinsys[i].solverData;
+        allocateHomotopyData(size-1, &(solverData->initHomotopyData));
+      } else {
+        nlsKinsolAllocate(size, &nonlinsys[i], data->simulationInfo->nlsLinearSolver);
+        solverData->ordinaryData = nonlinsys[i].solverData;
+      }
+      nonlinsys[i].solverData = (void*) solverData;
       break;
     case NLS_NEWTON:
-      allocateNewtonData(size, &nonlinsys[i].solverData);
+      solverData = (struct dataSolver*) malloc(sizeof(struct dataSolver));
+      if (nonlinsys[i].homotopySupport && (data->callback->useHomotopy == 2 || data->callback->useHomotopy == 3)) {
+        allocateNewtonData(size-1, &(solverData->ordinaryData));
+        allocateHomotopyData(size-1, &(solverData->initHomotopyData));
+      } else {
+        allocateNewtonData(size, &(solverData->ordinaryData));
+      }
+      nonlinsys[i].solverData = (void*) solverData;
       break;
 #endif
     case NLS_HOMOTOPY:
-      allocateHomotopyData(size, &nonlinsys[i].solverData);
+      if (nonlinsys[i].homotopySupport && (data->callback->useHomotopy == 2 || data->callback->useHomotopy == 3)) {
+        allocateHomotopyData(size-1, &nonlinsys[i].solverData);
+      } else {
+        allocateHomotopyData(size, &nonlinsys[i].solverData);
+      }
       break;
 #if !defined(OMC_MINIMAL_RUNTIME)
     case NLS_MIXED:
-      mixedSolverData = (struct dataNewtonAndHybrid*) malloc(sizeof(struct dataNewtonAndHybrid));
-      allocateHomotopyData(size, &(mixedSolverData->newtonData));
-
-      allocateHybrdData(size, &(mixedSolverData->hybridData));
-
+      mixedSolverData = (struct dataMixedSolver*) malloc(sizeof(struct dataMixedSolver));
+      if (nonlinsys[i].homotopySupport && (data->callback->useHomotopy == 2 || data->callback->useHomotopy == 3)) {
+        allocateHomotopyData(size-1, &(mixedSolverData->newtonHomotopyData));
+        allocateHybrdData(size-1, &(mixedSolverData->hybridData));
+      } else {
+        allocateHomotopyData(size, &(mixedSolverData->newtonHomotopyData));
+        allocateHybrdData(size, &(mixedSolverData->hybridData));
+      }
       nonlinsys[i].solverData = (void*) mixedSolverData;
-
       break;
 #endif
     default:
       throwStreamPrint(threadData, "unrecognized nonlinear solver");
     }
   }
-
   messageClose(LOG_NLS);
 
   TRACE_POP
@@ -463,7 +513,6 @@ int updateStaticDataOfNonlinearSystems(DATA *data, threadData_t *threadData)
   int i;
   int size;
   NONLINEAR_SYSTEM_DATA *nonlinsys = data->simulationInfo->nonlinearSystemData;
-  struct dataNewtonAndHybrid *mixedSolverData;
 
   infoStreamPrint(LOG_NLS, 1, "update static data of non-linear system solvers");
 
@@ -504,6 +553,7 @@ int freeNonlinearSystems(DATA *data, threadData_t *threadData)
     free(nonlinsys[i].max);
     freeValueList(nonlinsys[i].oldValueList, 1);
 
+
 #if !defined(OMC_MINIMAL_RUNTIME)
     if (data->simulationInfo->nlsCsvInfomation)
     {
@@ -517,24 +567,36 @@ int freeNonlinearSystems(DATA *data, threadData_t *threadData)
     {
 #if !defined(OMC_MINIMAL_RUNTIME)
     case NLS_HYBRID:
-      freeHybrdData(&nonlinsys[i].solverData);
+      freeHybrdData(&((struct dataSolver*) nonlinsys[i].solverData)->ordinaryData);
+      if (nonlinsys[i].homotopySupport && (data->callback->useHomotopy == 2 || data->callback->useHomotopy == 3)) {
+        freeHomotopyData(&((struct dataSolver*) nonlinsys[i].solverData)->initHomotopyData);
+      }
       free(nonlinsys[i].solverData);
       break;
     case NLS_KINSOL:
-      nlsKinsolFree(&nonlinsys[i].solverData);
+      if (nonlinsys[i].homotopySupport && (data->callback->useHomotopy == 2 || data->callback->useHomotopy == 3)) {
+        freeHomotopyData(&((struct dataSolver*) nonlinsys[i].solverData)->initHomotopyData);
+      } else {
+        nlsKinsolFree(&((struct dataSolver*) nonlinsys[i].solverData)->ordinaryData);
+      }
+      free(nonlinsys[i].solverData);
       break;
     case NLS_NEWTON:
-      freeNewtonData(&nonlinsys[i].solverData);
+      freeNewtonData(&((struct dataSolver*) nonlinsys[i].solverData)->ordinaryData);
+      if (nonlinsys[i].homotopySupport && (data->callback->useHomotopy == 2 || data->callback->useHomotopy == 3)) {
+        freeHomotopyData(&((struct dataSolver*) nonlinsys[i].solverData)->initHomotopyData);
+      }
       free(nonlinsys[i].solverData);
       break;
 #endif
     case NLS_HOMOTOPY:
       freeHomotopyData(&nonlinsys[i].solverData);
+      free(nonlinsys[i].solverData);
       break;
 #if !defined(OMC_MINIMAL_RUNTIME)
     case NLS_MIXED:
-      freeHomotopyData(&((struct dataNewtonAndHybrid*) nonlinsys[i].solverData)->newtonData);
-      freeHybrdData(&((struct dataNewtonAndHybrid*) nonlinsys[i].solverData)->hybridData);
+      freeHomotopyData(&((struct dataMixedSolver*) nonlinsys[i].solverData)->newtonHomotopyData);
+      freeHybrdData(&((struct dataMixedSolver*) nonlinsys[i].solverData)->hybridData);
       free(nonlinsys[i].solverData);
       break;
 #endif
@@ -570,6 +632,55 @@ void printNonLinearSystemSolvingStatistics(DATA *data, int sysNumber, int logLev
   messageClose(logLevel);
 }
 
+/*! \fn printNonLinearInitialInfo
+ *
+ *  This function prints information of an non-linear systems before an solving step.
+ *
+ *  \param [in]  [logName] log level in general LOG_NLS
+ *         [ref] [data]
+ *         [ref] [nonlinsys] index of corresponding non-linear system
+ */
+void printNonLinearInitialInfo(int logName, DATA* data, NONLINEAR_SYSTEM_DATA *nonlinsys)
+{
+  long i;
+
+  if (!ACTIVE_STREAM(logName)) return;
+  infoStreamPrint(logName, 1, "initial variable values:");
+
+  for(i=0; i<nonlinsys->size; i++)
+    infoStreamPrint(logName, 0, "[%2ld] %30s  = %16.8g\t\t nom = %16.8g", i+1,
+                    modelInfoGetEquation(&data->modelData->modelDataXml,nonlinsys->equationIndex).vars[i],
+                    nonlinsys->nlsx[i], nonlinsys->nominal[i]);
+  messageClose(logName);
+}
+
+/*! \fn printNonLinearFinishInfo
+ *
+ *  This function prints information of an non-linear systems after a solving step.
+ *
+ *  \param [in]  [logName] log level in general LOG_NLS
+ *         [ref] [data]
+ *         [ref] [nonlinsys] index of corresponding non-linear system
+ */
+void printNonLinearFinishInfo(int logName, DATA* data, NONLINEAR_SYSTEM_DATA *nonlinsys)
+{
+  long i;
+
+  if (!ACTIVE_STREAM(logName)) return;
+
+  infoStreamPrint(logName, 1, "Solution status: %s", nonlinsys->solved?"SOLVED":"FAILED");
+  infoStreamPrint(logName, 0, " number of iterations           : %ld", nonlinsys->numberOfIterations);
+  infoStreamPrint(logName, 0, " number of function evaluations : %ld", nonlinsys->numberOfFEval);
+  infoStreamPrint(logName, 0, " number of jacobian evaluations : %ld", nonlinsys->numberOfJEval);
+  infoStreamPrint(logName, 0, "solution values:");
+  for(i=0; i<nonlinsys->size; i++)
+    infoStreamPrint(logName, 0, "[%2ld] %30s  = %16.8g", i+1,
+                    modelInfoGetEquation(&data->modelData->modelDataXml,nonlinsys->equationIndex).vars[i],
+                    nonlinsys->nlsx[i]);
+
+  messageClose(logName);
+}
+
 /*! \fn getInitialGuess
  *
  *  This function writes initial guess to nonlinsys->nlsx and nonlinsys->nlsOld.
@@ -698,19 +809,28 @@ int solveNLS(DATA *data, threadData_t *threadData, int sysNumber)
   int success = 0, constraintsSatisfied = 1;
   NONLINEAR_SYSTEM_DATA* nonlinsys = &(data->simulationInfo->nonlinearSystemData[sysNumber]);
   int casualTearingSet = nonlinsys->strictTearingFunctionCall != NULL;
-  struct dataNewtonAndHybrid *mixedSolverData;
+  struct dataSolver *solverData;
+  struct dataMixedSolver *mixedSolverData;
 
   /* use the selected solver for solving nonlinear system */
   switch(data->simulationInfo->nlsMethod)
   {
 #if !defined(OMC_MINIMAL_RUNTIME)
   case NLS_HYBRID:
+    solverData = nonlinsys->solverData;
+    nonlinsys->solverData = solverData->ordinaryData;
     success = solveHybrd(data, threadData, sysNumber);
+    nonlinsys->solverData = solverData;
     break;
   case NLS_KINSOL:
+    solverData = nonlinsys->solverData;
+    nonlinsys->solverData = solverData->ordinaryData;
     success = nlsKinsolSolve(data, threadData, sysNumber);
+    nonlinsys->solverData = solverData;
     break;
   case NLS_NEWTON:
+    solverData = nonlinsys->solverData;
+    nonlinsys->solverData = solverData->ordinaryData;
     success = solveNewton(data, threadData, sysNumber);
     /* check if solution process was successful, if not use alternative tearing set if available (dynamic tearing)*/
     if (!success && casualTearingSet){
@@ -718,6 +838,7 @@ int solveNLS(DATA *data, threadData_t *threadData, int sysNumber)
       success = nonlinsys->strictTearingFunctionCall(data, threadData);
       if (success) success=2;
     }
+    nonlinsys->solverData = solverData;
     break;
 #endif
   case NLS_HOMOTOPY:
@@ -726,22 +847,18 @@ int solveNLS(DATA *data, threadData_t *threadData, int sysNumber)
 #if !defined(OMC_MINIMAL_RUNTIME)
   case NLS_MIXED:
     mixedSolverData = nonlinsys->solverData;
-    nonlinsys->solverData = mixedSolverData->newtonData;
-
+    nonlinsys->solverData = mixedSolverData->newtonHomotopyData;
 
     /* try to solve the system if it is the strict set, only try to solve the casual set if the constraints are satisfied */
     if ((!casualTearingSet) || constraintsSatisfied)
       success = solveHomotopy(data, threadData, sysNumber);
 
-
     /* check if solution process was successful, if not use alternative tearing set if available (dynamic tearing)*/
     if (!success && casualTearingSet){
       debugString(LOG_DT, "Solving the casual tearing set failed! Now the strict tearing set is used.");
       success = nonlinsys->strictTearingFunctionCall(data, threadData);
       if (success){
         success=2;
-        /* update iteration variables of the casual set*/
-        nonlinsys->getIterationVars(data, nonlinsys->nlsx);
       }
     }
 
@@ -749,6 +866,58 @@ int solveNLS(DATA *data, threadData_t *threadData, int sysNumber)
       nonlinsys->solverData = mixedSolverData->hybridData;
       success = solveHybrd(data, threadData, sysNumber);
     }
+
+    /* update iteration variables of nonlinsys->nlsx */
+    if (success){
+      nonlinsys->getIterationVars(data, nonlinsys->nlsx);
+    }
+
+    nonlinsys->solverData = mixedSolverData;
+    break;
+#endif
+  default:
+    throwStreamPrint(threadData, "unrecognized nonlinear solver");
+  }
+
+  return success;
+}
+
+/*! \fn solve system with homotopy solver
+ *
+ *  \param [in]  [data]
+ *  \param [in]  [threadData]
+ *  \param [in]  [sysNumber] index of corresponding non-linear system
+ *
+ *  \author ptaeuber
+ */
+int solveWithInitHomotopy(DATA *data, threadData_t *threadData, int sysNumber)
+{
+  int success = 0;
+  NONLINEAR_SYSTEM_DATA* nonlinsys = &(data->simulationInfo->nonlinearSystemData[sysNumber]);
+  struct dataSolver *solverData;
+  struct dataMixedSolver *mixedSolverData;
+
+  /* use the homotopy solver for solving the initial system */
+  switch(data->simulationInfo->nlsMethod)
+  {
+#if !defined(OMC_MINIMAL_RUNTIME)
+  case NLS_HYBRID:
+  case NLS_KINSOL:
+  case NLS_NEWTON:
+    solverData = nonlinsys->solverData;
+    nonlinsys->solverData = solverData->initHomotopyData;
+    success = solveHomotopy(data, threadData, sysNumber);
+    nonlinsys->solverData = solverData;
+    break;
+#endif
+  case NLS_HOMOTOPY:
+    success = solveHomotopy(data, threadData, sysNumber);
+    break;
+#if !defined(OMC_MINIMAL_RUNTIME)
+  case NLS_MIXED:
+    mixedSolverData = nonlinsys->solverData;
+    nonlinsys->solverData = mixedSolverData->newtonHomotopyData;
+    success = solveHomotopy(data, threadData, sysNumber);
     nonlinsys->solverData = mixedSolverData;
     break;
 #endif
@@ -764,6 +933,8 @@ int solveNLS(DATA *data, threadData_t *threadData, int sysNumber)
  *  \param [in]  [data]
  *  \param [in]  [threadData]
  *  \param [in]  [sysNumber] index of corresponding non-linear system
+ *
+ *  \author ptaeuber
  */
 int solve_nonlinear_system(DATA *data, threadData_t *threadData, int sysNumber)
 {
@@ -772,11 +943,21 @@ int solve_nonlinear_system(DATA *data, threadData_t *threadData, int sysNumber)
   NONLINEAR_SYSTEM_DATA* nonlinsys = &(data->simulationInfo->nonlinearSystemData[sysNumber]);
   int casualTearingSet = nonlinsys->strictTearingFunctionCall != NULL;
   int step;
-  int lambda_steps = 1;
+  int equidistantHomotopy = 0;
+  int solveWithHomotopySolver = 0;
+  int homotopyDeactivated = 0;
   int j;
+  int nlsLs;
+  int kinsol = 0;
+  struct dataSolver *solverData;
+  struct dataMixedSolver *mixedSolverData;
   char buffer[4096];
   FILE *pFile = NULL;
 
+#if !defined(OMC_MINIMAL_RUNTIME)
+  kinsol = (data->simulationInfo->nlsMethod == NLS_KINSOL);
+#endif
+
   data->simulationInfo->currentNonlinearSystemIndex = sysNumber;
 
   /* enable to avoid division by zero */
@@ -786,8 +967,8 @@ int solve_nonlinear_system(DATA *data, threadData_t *threadData, int sysNumber)
   /* performance measurement */
   rt_ext_tp_tick(&nonlinsys->totalTimeClock);
 
+  infoStreamPrint(LOG_NLS_EXTRAPOLATE, 1, "Nonlinear system %ld dump LOG_NLS_EXTRAPOLATE", nonlinsys->equationIndex);
   /* grab the initial guess */
-  infoStreamPrint(LOG_NLS_EXTRAPOLATE, 1, "############ Start new iteration for system %ld at time %g ############", nonlinsys->equationIndex, data->localData[0]->timeValue);
   /* if last solving is too long ago use just old values  */
   if (fabs(data->localData[0]->timeValue - nonlinsys->lastTimeSolved) < 5*data->simulationInfo->stepSize || casualTearingSet)
   {
@@ -804,6 +985,10 @@ int solve_nonlinear_system(DATA *data, threadData_t *threadData, int sysNumber)
     constraintsSatisfied = updateInnerEquation(dataAndThreadData, sysNumber, 1);
   }
 
+  /* print debug initial information */
+  infoStreamPrint(LOG_NLS, 1, "############ Solve nonlinear system %ld at time %g ############", nonlinsys->equationIndex, data->localData[0]->timeValue);
+  printNonLinearInitialInfo(LOG_NLS, data, nonlinsys);
+
   /* try */
 #ifndef OMC_EMCC
   MMC_TRY_INTERNAL(simulationJumpBuffer)
@@ -813,59 +998,128 @@ int solve_nonlinear_system(DATA *data, threadData_t *threadData, int sysNumber)
   saveJumpState = threadData->currentErrorStage;
   threadData->currentErrorStage = ERROR_NONLINEARSOLVER;
 
-  if(data->simulationInfo->initial && nonlinsys->homotopySupport &&
-     (data->callback->useHomotopy == 0) && init_lambda_steps > 1)
-    lambda_steps = init_lambda_steps;
+  equidistantHomotopy = data->simulationInfo->initial
+                        && nonlinsys->homotopySupport
+                        && (data->callback->useHomotopy == 0 && init_lambda_steps > 1);
 
-#if !defined(OMC_NO_FILESYSTEM)
-  if(lambda_steps > 1 && ACTIVE_STREAM(LOG_INIT))
-  {
-    sprintf(buffer, "%s_homotopy_nls_%d.csv", data->modelData->modelFilePrefix, sysNumber);
-    infoStreamPrint(LOG_INIT, 0, "The homotopy path of system %d will be exported to %s.", sysNumber, buffer);
-    pFile = fopen(buffer, "wt");
-    fprintf(pFile, "\"sep=,\"\n%s", "lambda");
-    for(j=0; j<nonlinsys->size; ++j)
-      fprintf(pFile, ",%s", modelInfoGetEquation(&data->modelData->modelDataXml, nonlinsys->equationIndex).vars[j]);
-    fprintf(pFile, "\n");
+  solveWithHomotopySolver = data->simulationInfo->initial
+                            && nonlinsys->homotopySupport
+                            && (data->callback->useHomotopy == 2 || data->callback->useHomotopy == 3);
+
+  homotopyDeactivated = !data->simulationInfo->initial           // Not an initialization system
+                        || !nonlinsys->homotopySupport           // There is no homotopy in this component
+                        || (data->callback->useHomotopy == 1)    // Equidistant homotopy is performed globally in symbolic_initialization()
+                        || (data->callback->useHomotopy == 0     // Equidistant local homotopy is selected but homotopy is deactivated ...
+                            && init_lambda_steps <= 1);          // ... by the number of steps
+
+  nonlinsys->solved = 0;
+  nonlinsys->initHomotopy = 0;
+
+  /* If homotopy is deactivated in this place or flag homotopyOnFirstTry is not set,
+     solve the system with the selected solver */
+  if (homotopyDeactivated || !omc_flag[FLAG_HOMOTOPY_ON_FIRST_TRY]) {
+    if (solveWithHomotopySolver && kinsol) {
+      infoStreamPrint(LOG_INIT, 0, "Automatically set -homotopyOnFirstTry, because trying without homotopy first is not supported for the local global approach in combination with KINSOL.");
+    } else {
+      if (!homotopyDeactivated && !omc_flag[FLAG_HOMOTOPY_ON_FIRST_TRY])
+        infoStreamPrint(LOG_INIT, 0, "Try to solve nonlinear initial system %d without homotopy first.", sysNumber);
+
+      data->simulationInfo->lambda = 1.0;
+      /* SOLVE! */
+      nonlinsys->solved = solveNLS(data, threadData, sysNumber);
+    }
   }
+
+  /* The following cases are only valid for initial systems with homotopy */
+  /* **********************************************************************/
+
+  /* If the adaptive local/global homotopy approach is activated and trying without homotopy failed or is not wanted,
+     use the HOMOTOPY SOLVER */
+  if (solveWithHomotopySolver && !nonlinsys->solved) {
+    if (!omc_flag[FLAG_HOMOTOPY_ON_FIRST_TRY] && !kinsol)
+      warningStreamPrint(LOG_ASSERT, 0, "Failed to solve the initial system %d without homotopy method.", sysNumber);
+    data->simulationInfo->lambda = 0.0;
+    if (data->callback->useHomotopy == 3) {
+      // First solve the lambda0-system separately
+      infoStreamPrint(LOG_INIT, 0, "Local homotopy with adaptive step size started for nonlinear system %d.", sysNumber);
+      infoStreamPrint(LOG_INIT, 1, "homotopy process\n---------------------------");
+      infoStreamPrint(LOG_INIT, 0, "solve lambda0-system");
+      nonlinsys->homotopySupport = 0;
+      if (!kinsol) {
+        nonlinsys->solved = solveNLS(data, threadData, sysNumber);
+      } else {
+        nlsLs = data->simulationInfo->nlsLinearSolver;
+        data->simulationInfo->nlsLinearSolver = NLS_LS_LAPACK;
+        nonlinsys->solved = solveWithInitHomotopy(data, threadData, sysNumber);
+        data->simulationInfo->nlsLinearSolver = nlsLs;
+      }
+      nonlinsys->homotopySupport = 1;
+      infoStreamPrint(LOG_INIT, 0, "solving lambda0-system done with%s success\n---------------------------", nonlinsys->solved ? "" : " no");
+      messageClose(LOG_INIT);
+    }
+    /* SOLVE! */
+    if (data->callback->useHomotopy == 2 || nonlinsys->solved) {
+      infoStreamPrint(LOG_INIT, 0, "run along the homotopy path and solve the actual system");
+      nonlinsys->initHomotopy = 1;
+      nonlinsys->solved = solveWithInitHomotopy(data, threadData, sysNumber);
+    }
+  }
+
+  /* If equidistant local homotopy is activated and trying without homotopy failed or is not wanted,
+     use EQUIDISTANT LOCAL HOMOTOPY */
+  if (equidistantHomotopy && !nonlinsys->solved) {
+    if (!omc_flag[FLAG_HOMOTOPY_ON_FIRST_TRY])
+      warningStreamPrint(LOG_ASSERT, 0, "Failed to solve the initial system %d without homotopy method. The local homotopy method with equidistant step size is used now.", sysNumber);
+    else
+      infoStreamPrint(LOG_INIT, 0, "Local homotopy with equidistant step size started for nonlinear system %d.", sysNumber);
+#if !defined(OMC_NO_FILESYSTEM)
+    const char sep[] = ",";
+    if(ACTIVE_STREAM(LOG_INIT))
+    {
+      sprintf(buffer, "%s_nonlinsys%d_equidistant_local_homotopy.csv", data->modelData->modelFilePrefix, sysNumber);
+      infoStreamPrint(LOG_INIT, 0, "The homotopy path of system %d will be exported to %s.", sysNumber, buffer);
+      pFile = omc_fopen(buffer, "wt");
+      fprintf(pFile, "\"sep=%s\"\n%s", sep, "\"lambda\"");
+      for(j=0; j<nonlinsys->size; ++j)
+        fprintf(pFile, "%s\"%s\"", sep, modelInfoGetEquation(&data->modelData->modelDataXml, nonlinsys->equationIndex).vars[j]);
+      fprintf(pFile, "\n");
+    }
 #endif
 
-  for(step=0; step<lambda_steps-1; ++step)
-  {
-    data->simulationInfo->lambda = ((double)step)/(lambda_steps-1);
-    infoStreamPrint(LOG_INIT, 0, "[system %d] homotopy parameter lambda = %g", sysNumber, data->simulationInfo->lambda);
-    solveNLS(data, threadData, sysNumber);
+    for(step=0; step<init_lambda_steps; ++step)
+    {
+      data->simulationInfo->lambda = ((double)step)/(init_lambda_steps-1);
+
+      if (data->simulationInfo->lambda > 1.0) {
+        data->simulationInfo->lambda = 1.0;
+      }
+
+      infoStreamPrint(LOG_INIT, 0, "[system %d] homotopy parameter lambda = %g", sysNumber, data->simulationInfo->lambda);
+      /* SOLVE! */
+      nonlinsys->solved = solveNLS(data, threadData, sysNumber);
+      if (!nonlinsys->solved) break;
+
+#if !defined(OMC_NO_FILESYSTEM)
+      if(ACTIVE_STREAM(LOG_INIT))
+      {
+        infoStreamPrint(LOG_INIT, 0, "[system %d] homotopy parameter lambda = %g done\n---------------------------", sysNumber, data->simulationInfo->lambda);
+        fprintf(pFile, "%.16g", data->simulationInfo->lambda);
+        for(j=0; j<nonlinsys->size; ++j)
+          fprintf(pFile, "%s%.16g", sep, nonlinsys->nlsx[j]);
+        fprintf(pFile, "\n");
+      }
+#endif
+    }
 
 #if !defined(OMC_NO_FILESYSTEM)
     if(ACTIVE_STREAM(LOG_INIT))
     {
-      infoStreamPrint(LOG_INIT, 0, "[system %d] homotopy parameter lambda = %g done\n---------------------------", sysNumber, data->simulationInfo->lambda);
-      fprintf(pFile, "%.16g", data->simulationInfo->lambda);
-      for(j=0; j<nonlinsys->size; ++j)
-        fprintf(pFile, ",%.16g", nonlinsys->nlsx[j]);
-      fprintf(pFile, "\n");
+      fclose(pFile);
     }
 #endif
+    data->simulationInfo->homotopySteps += init_lambda_steps;
   }
 
-  if(data->callback->useHomotopy == 0)
-    data->simulationInfo->lambda = 1.0;
-
-  /* SOLVE! */
-  nonlinsys->solved = solveNLS(data, threadData, sysNumber);
-
-#if !defined(OMC_NO_FILESYSTEM)
-  if(lambda_steps > 1 && ACTIVE_STREAM(LOG_INIT))
-  {
-    infoStreamPrint(LOG_INIT, 0, "[system %d] homotopy parameter lambda = %g done\n---------------------------", sysNumber, data->simulationInfo->lambda);
-    fprintf(pFile, "%.16g", data->simulationInfo->lambda);
-    for(j=0; j<nonlinsys->size; ++j)
-      fprintf(pFile, ",%.16g", nonlinsys->nlsx[j]);
-    fprintf(pFile, "\n");
-    fclose(pFile);
-  }
-#endif
-
   /* handle asserts */
   threadData->currentErrorStage = saveJumpState;
 
@@ -880,6 +1134,8 @@ int solve_nonlinear_system(DATA *data, threadData_t *threadData, int sysNumber)
   {
     nonlinsys->lastTimeSolved = data->localData[0]->timeValue;
   }
+  printNonLinearFinishInfo(LOG_NLS, data, nonlinsys);
+  messageClose(LOG_NLS);
 
 
   /* enable to avoid division by zero */
@@ -953,10 +1209,10 @@ int check_nonlinear_solution(DATA *data, int printFailingSystems, int sysNumber)
   {
     int index = nonlinsys[i].equationIndex, indexes[2] = {1,index};
     if (!printFailingSystems) return 1;
-    warningStreamPrintWithEquationIndexes(LOG_NLS, 1, indexes, "nonlinear system %d fails: at t=%g", index, data->localData[0]->timeValue);
+    warningStreamPrintWithEquationIndexes(LOG_NLS, 0, indexes, "nonlinear system %d fails: at t=%g", index, data->localData[0]->timeValue);
     if(data->simulationInfo->initial)
     {
-      warningStreamPrint(LOG_NLS_V, 0, "proper start-values for some of the following iteration variables might help");
+      warningStreamPrint(LOG_INIT, 1, "proper start-values for some of the following iteration variables might help");
     }
     for(j=0; j<modelInfoGetEquation(&data->modelData->modelDataXml, (nonlinsys[i]).equationIndex).numVar; ++j) {
       int done=0;
@@ -967,7 +1223,7 @@ int check_nonlinear_solution(DATA *data, int printFailingSystems, int sysNumber)
         if (!strcmp(mData->realVarsData[k].info.name, modelInfoGetEquation(&data->modelData->modelDataXml, (nonlinsys[i]).equationIndex).vars[j]))
         {
         done = 1;
-        warningStreamPrint(LOG_NLS_V, 0, "[%ld] Real %s(start=%g, nominal=%g)", j+1,
+        warningStreamPrint(LOG_INIT, 0, "[%ld] Real %s(start=%g, nominal=%g)", j+1,
                                      mData->realVarsData[k].info.name,
                                      mData->realVarsData[k].attribute.start,
                                      mData->realVarsData[k].attribute.nominal);
@@ -975,10 +1231,10 @@ int check_nonlinear_solution(DATA *data, int printFailingSystems, int sysNumber)
       }
       if (!done)
       {
-        warningStreamPrint(LOG_NLS_V, 0, "[%ld] Real %s(start=?, nominal=?)", j+1, modelInfoGetEquation(&data->modelData->modelDataXml, (nonlinsys[i]).equationIndex).vars[j]);
+        warningStreamPrint(LOG_INIT, 0, "[%ld] Real %s(start=?, nominal=?)", j+1, modelInfoGetEquation(&data->modelData->modelDataXml, (nonlinsys[i]).equationIndex).vars[j]);
       }
     }
-    messageCloseWarning(LOG_NLS);
+    messageCloseWarning(LOG_INIT);
     return 1;
   }
   if(nonlinsys[i].solved == 2)
