@@ -51,15 +51,32 @@ enum _FLAG
   FLAG_DELTA_X_SOLVER,
   FLAG_EMBEDDED_SERVER,
   FLAG_EMBEDDED_SERVER_PORT,
+  FLAG_MAT_SYNC,
   FLAG_EMIT_PROTECTED,
+  FLAG_DATA_RECONCILE_Eps,
   FLAG_F,
   FLAG_HELP,
+  FLAG_HOMOTOPY_ADAPT_BEND,
+  FLAG_HOMOTOPY_BACKTRACE_STRATEGY,
+  FLAG_HOMOTOPY_H_EPS,
+  FLAG_HOMOTOPY_MAX_LAMBDA_STEPS,
+  FLAG_HOMOTOPY_MAX_NEWTON_STEPS,
+  FLAG_HOMOTOPY_MAX_TRIES,
+  FLAG_HOMOTOPY_NEG_START_DIR,
   FLAG_HOMOTOPY_ON_FIRST_TRY,
+  FLAG_HOMOTOPY_TAU_DEC_FACTOR,
+  FLAG_HOMOTOPY_TAU_DEC_FACTOR_PRED,
+  FLAG_HOMOTOPY_TAU_INC_FACTOR,
+  FLAG_HOMOTOPY_TAU_INC_THRESHOLD,
+  FLAG_HOMOTOPY_TAU_MAX,
+  FLAG_HOMOTOPY_TAU_MIN,
+  FLAG_HOMOTOPY_TAU_START,
   FLAG_IDA_MAXERRORTESTFAIL,
   FLAG_IDA_MAXNONLINITERS,
   FLAG_IDA_MAXCONVFAILS,
   FLAG_IDA_NONLINCONVCOEF,
   FLAG_IDA_LS,
+  FLAG_IDA_SCALING,
   FLAG_IDAS,
   FLAG_IGNORE_HIDERESULT,
   FLAG_IIF,
@@ -72,6 +89,7 @@ enum _FLAG
   FLAG_INPUT_CSV,
   FLAG_INPUT_FILE,
   FLAG_INPUT_FILE_STATES,
+  FLAG_INPUT_PATH,
   FLAG_IPOPT_HESSE,
   FLAG_IPOPT_INIT,
   FLAG_IPOPT_JAC,
@@ -93,6 +111,7 @@ enum _FLAG
   FLAG_MAX_STEP_SIZE,
   FLAG_MEASURETIMEPLOTFORMAT,
   FLAG_NEWTON_FTOL,
+  FLAG_NEWTON_MAX_STEP_FACTOR,
   FLAG_NEWTON_XTOL,
   FLAG_NEWTON_STRATEGY,
   FLAG_NLS,
@@ -107,20 +126,25 @@ enum _FLAG
   FLAG_NOEVENTEMIT,
   FLAG_NO_RESTART,
   FLAG_NO_ROOTFINDING,
+  FLAG_NO_SCALING,
   FLAG_NO_SUPPRESS_ALG,
   FLAG_OPTDEBUGEJAC,
   FLAG_OPTIMIZER_NP,
   FLAG_OPTIMIZER_TGRID,
   FLAG_OUTPUT,
+  FLAG_OUTPUT_PATH,
   FLAG_OVERRIDE,
   FLAG_OVERRIDE_FILE,
   FLAG_PORT,
   FLAG_R,
+  FLAG_DATA_RECONCILE,
   FLAG_RT,
   FLAG_S,
+  FLAG_SINGLE_PRECISION,
   FLAG_SOLVER_STEPS,
   FLAG_STEADY_STATE,
   FLAG_STEADY_STATE_TOL,
+  FLAG_DATA_RECONCILE_Sx,
   FLAG_UP_HESSIAN,
   FLAG_W,
 
@@ -185,6 +209,8 @@ enum LINEAR_SOLVER
   LS_LAPACK,
 #if !defined(OMC_MINIMAL_RUNTIME)
   LS_LIS,
+#else
+  LS_LIS_NOT_AVAILABLE,
 #endif
   LS_KLU,
   LS_UMFPACK,
@@ -193,24 +219,26 @@ enum LINEAR_SOLVER
 
   LS_MAX
 };
+extern const char *LS_NAME[LS_MAX];
+extern const char *LS_DESC[LS_MAX];
 
 enum LINEAR_SPARSE_SOLVER
 {
   LSS_NONE = 0,
 
+  LSS_DEFAULT,
 #if !defined(OMC_MINIMAL_RUNTIME)
   LSS_LIS,
+#else
+  LSS_LIS_NOT_AVAILABLE,
 #endif
   LSS_KLU,
   LSS_UMFPACK,
-
   LSS_MAX
 };
 
-extern const char *LS_NAME[LS_MAX];
-extern const char *LS_DESC[LS_MAX];
-extern const char *LSS_NAME[LS_MAX];
-extern const char *LSS_DESC[LS_MAX];
+extern const char *LSS_NAME[LSS_MAX];
+extern const char *LSS_DESC[LSS_MAX];
 
 enum NONLINEAR_SOLVER
 {
@@ -219,14 +247,21 @@ enum NONLINEAR_SOLVER
 #if !defined(OMC_MINIMAL_RUNTIME)
   NLS_HYBRID,
   NLS_KINSOL,
-#endif
   NLS_NEWTON,
-  NLS_HOMOTOPY,
   NLS_MIXED,
+#else
+  NLS_HYBRID_DOESNT_EXIST,
+  NLS_KINSOL_DOESNT_EXIST,
+  NLS_NEWTON_DOESNT_EXIST,
+  NLS_MIXED_DOESNT_EXIST,
+#endif
+  NLS_HOMOTOPY,
 
   NLS_MAX
 };
 
+extern const char *NLS_NAME[NLS_MAX];
+extern const char *NLS_DESC[NLS_MAX];
 
 enum NEWTON_STRATEGY
 {
@@ -241,9 +276,6 @@ enum NEWTON_STRATEGY
   NEWTON_MAX
 };
 
-extern const char *NLS_NAME[NLS_MAX+1];
-extern const char *NLS_DESC[NLS_MAX+1];
-
 extern const char *NEWTONSTRATEGY_NAME[NEWTON_MAX];
 extern const char *NEWTONSTRATEGY_DESC[NEWTON_MAX];
 
@@ -256,7 +288,6 @@ enum JACOBIAN_METHOD
   COLOREDSYMJAC,
   NUMJAC,
   SYMJAC,
-  KLUSPARSE,
 
   JAC_MAX
 };
@@ -284,6 +315,8 @@ enum NLS_LS
 {
   NLS_LS_UNKNOWN = 0,
 
+  NLS_LS_DEFAULT,
+
   NLS_LS_TOTALPIVOT,
   NLS_LS_LAPACK,
   NLS_LS_KLU,
@@ -307,6 +340,19 @@ enum IMPRK_LS
 extern const char *IMPRK_LS_METHOD[IMPRK_LS_MAX];
 extern const char *IMPRK_LS_METHOD_DESC[IMPRK_LS_MAX];
 
+enum HOMOTOPY_BACKTRACE_STRATEGY
+{
+  HOM_BACK_STRAT_NONE = 0,
+
+  HOM_BACK_STRAT_FIX,
+  HOM_BACK_STRAT_ORTHOGONAL,
+
+  HOM_BACK_STRAT_MAX
+};
+
+extern const char *HOM_BACK_STRAT_NAME[HOM_BACK_STRAT_MAX];
+extern const char *HOM_BACK_STRAT_DESC[HOM_BACK_STRAT_MAX];
+
 
 #if defined(__cplusplus)
   }
