@@ -1,7 +1,7 @@
 /*
  * This file is part of OpenModelica.
  *
- * Copyright (c) 1998-2014, Open Source Modelica Consortium (OSMC),
+ * Copyright (c) 1998-CurrentYear, Open Source Modelica Consortium (OSMC),
  * c/o Linköpings universitet, Department of Computer and Information Science,
  * SE-58183 Linköping, Sweden.
  *
@@ -46,6 +46,7 @@
 #include "Annotations/EllipseAnnotation.h"
 #include "Annotations/TextAnnotation.h"
 #include "Annotations/BitmapAnnotation.h"
+#include "OMS/OMSProxy.h"
 
 class OMCProxy;
 class GraphicsScene;
@@ -66,14 +67,13 @@ public:
   ComponentInfo(ComponentInfo *pComponentInfo, QObject *pParent = 0);
   void updateComponentInfo(const ComponentInfo *pComponentInfo);
   void parseComponentInfoString(QString value);
-  void fetchModifiers(OMCProxy *pOMCProxy, QString className, Component *pComponent);
-  void fetchParameterValue(OMCProxy *pOMCProxy, QString className);
+  void fetchParameterValue(OMCProxy *pOMCProxy, const QString &className);
   void applyDefaultPrefixes(QString defaultPrefixes);
-  void setClassName(QString className) {mClassName = className;}
+  void setClassName(const QString &className) {mClassName = className;}
   QString getClassName() const {return mClassName;}
-  void setName(QString name) {mName = name;}
+  void setName(const QString &name) {mName = name;}
   QString getName() const {return mName;}
-  void setComment(QString comment) {mComment = comment;}
+  void setComment(const QString &comment) {mComment = comment;}
   QString getComment() const {return StringHandler::removeFirstLastQuotes(mComment);}
   void setProtected(bool protect) {mIsProtected = protect;}
   bool getProtected() const {return mIsProtected;}
@@ -85,41 +85,50 @@ public:
   bool getStream() const {return mIsStream;}
   void setReplaceable(bool replaceable) {mIsReplaceable = replaceable;}
   bool getReplaceable() const {return mIsReplaceable;}
-  void setVariablity(QString variability) {mVariability = variability;}
+  void setVariablity(const QString &variability) {mVariability = variability;}
   QString getVariablity() const {return mVariability;}
   void setInner(bool inner) {mIsInner = inner;}
   bool getInner() const {return mIsInner;}
   void setOuter(bool outer) {mIsOuter = outer;}
   bool getOuter() const {return mIsOuter;}
-  void setCausality(QString causality) {mCasuality = causality;}
+  void setCausality(const QString &causality) {mCasuality = causality;}
   QString getCausality() const {return mCasuality;}
-  void setArrayIndex(QString arrayIndex);
+  void setArrayIndex(const QString &arrayIndex);
   QString getArrayIndex() const {return mArrayIndex;}
+  int getArrayIndexAsNumber(bool *ok) const;
   bool isArray() const {return mIsArray;}
   bool isModifiersLoaded() const {return mModifiersLoaded;}
+  void setModifiersLoaded(bool modifiersLoaded) {mModifiersLoaded = modifiersLoaded;}
   void setModifiersMap(QMap<QString, QString> modifiersMap) {mModifiersMap = modifiersMap;}
   QMap<QString, QString> getModifiersMapWithoutFetching() const {return mModifiersMap;}
   QMap<QString, QString> getModifiersMap(OMCProxy *pOMCProxy, QString className, Component *pComponent);
   bool isParameterValueLoaded() const {return mParameterValueLoaded;}
-  void setParameterValue(QString parameterValue) {mParameterValue = parameterValue;}
+  void setParameterValue(const QString &parameterValue) {mParameterValue = parameterValue;}
   QString getParameterValueWithoutFetching() const {return mParameterValue;}
-  QString getParameterValue(OMCProxy *pOMCProxy, QString className);
-  // MetaModel attributes
-  void setStartCommand(QString startCommand) {mStartCommand = startCommand;}
+  QString getParameterValue(OMCProxy *pOMCProxy, const QString &className);
+  // CompositeModel attributes
+  void setStartCommand(const QString &startCommand) {mStartCommand = startCommand;}
   QString getStartCommand() const {return mStartCommand;}
   void setExactStep(bool exactStep) {mExactStep = exactStep;}
   bool getExactStep() const {return mExactStep;}
-  void setModelFile(QString modelFile) {mModelFile = modelFile;}
+  void setModelFile(const QString &modelFile) {mModelFile = modelFile;}
   QString getModelFile() const {return mModelFile;}
-  void setGeometryFile(QString geometryFile) {mGeometryFile = geometryFile;}
+  void setGeometryFile(const QString &geometryFile) {mGeometryFile = geometryFile;}
   QString getGeometryFile() const {return mGeometryFile;}
-  void setPosition(QString position) {mPosition = position;}
+  void setPosition(const QString &position) {mPosition = position;}
   QString getPosition() const {return mPosition;}
-  void setAngle321(QString angle321) {mAngle321 = angle321;}
+  void setAngle321(const QString &angle321) {mAngle321 = angle321;}
   QString getAngle321() const {return mAngle321;}
+  void setDimensions(int dimensions) {mDimensions = dimensions;}
+  int getDimensions() const {return mDimensions;}
+  void setTLMCausality(const QString &causality) {mTLMCausality = causality;}
+  QString getTLMCausality() const {return mTLMCausality;}
+  void setDomain(const QString &domain) {mDomain = domain;}
+  QString getDomain() const {return mDomain;}
   // operator overloading
   bool operator==(const ComponentInfo &componentInfo) const;
   bool operator!=(const ComponentInfo &componentInfo) const;
+  QString getHTMLDescription() const;
 private:
   QString mClassName;
   QString mName;
@@ -141,14 +150,18 @@ private:
   QMap<QString, QString> mModifiersMap;
   bool mParameterValueLoaded;
   QString mParameterValue;
-  // MetaModel attributes
+  // CompositeModel attributes
   QString mStartCommand;
   bool mExactStep;
   QString mModelFile;
   QString mGeometryFile;
   QString mPosition;
   QString mAngle321;
+  int mDimensions;
+  QString mTLMCausality;
+  QString mDomain;
 
+  void fetchModifiers(OMCProxy *pOMCProxy, QString className, Component *pComponent);
   bool isModiferClassRecord(QString modifierName, Component *pComponent);
 };
 
@@ -162,8 +175,8 @@ public:
     Extend,  /* Inherited Component. */
     Port  /* Port Component. */
   };
-  Component(QString name, LibraryTreeItem *pLibraryTreeItem, QString transformation, QPointF position, QStringList dialogAnnotation,
-            ComponentInfo *pComponentInfo, GraphicsView *pGraphicsView);
+  Component(QString name, LibraryTreeItem *pLibraryTreeItem, QString annotation, QPointF position, ComponentInfo *pComponentInfo,
+            GraphicsView *pGraphicsView);
   Component(LibraryTreeItem *pLibraryTreeItem, Component *pParentComponent);
   Component(Component *pComponent, Component *pParentComponent, Component *pRootParentComponent);
   Component(Component *pComponent, GraphicsView *pGraphicsView);
@@ -172,9 +185,9 @@ public:
   bool isInheritedComponent() {return mIsInheritedComponent;}
   bool hasShapeAnnotation(Component *pComponent);
   bool hasNonExistingClass();
-  QRectF boundingRect() const;
+  QRectF boundingRect() const override;
   QRectF itemsBoundingRect();
-  void paint(QPainter *painter, const QStyleOptionGraphicsItem *option, QWidget *widget = 0);
+  void paint(QPainter *painter, const QStyleOptionGraphicsItem *option, QWidget *widget = 0) override;
   LibraryTreeItem* getLibraryTreeItem() {return mpLibraryTreeItem;}
   QString getName() {return mpComponentInfo->getName();}
   GraphicsView* getGraphicsView() {return mpGraphicsView;}
@@ -185,13 +198,16 @@ public:
   QString getTransformationString() {return mTransformationString;}
   void setDialogAnnotation(QStringList dialogAnnotation) {mDialogAnnotation = dialogAnnotation;}
   QStringList getDialogAnnotation() {return mDialogAnnotation;}
+  void setChoicesAnnotation(QStringList choicesAnnotation) {mChoicesAnnotation = choicesAnnotation;}
+  QStringList getChoicesAnnotation() {return mChoicesAnnotation;}
   CoOrdinateSystem getCoOrdinateSystem() const;
   OriginItem* getOriginItem() {return mpOriginItem;}
   QAction* getParametersAction() {return mpParametersAction;}
+  QAction* getFetchInterfaceDataAction() {return mpFetchInterfaceDataAction;}
   QAction* getAttributesAction() {return mpAttributesAction;}
   QAction* getOpenClassAction() {return mpOpenClassAction;}
-  QAction* getViewDocumentationAction() {return mpViewDocumentationAction;}
   QAction* getSubModelAttributesAction() {return mpSubModelAttributesAction;}
+  QAction* getElementPropertiesAction() {return mpElementPropertiesAction;}
   ComponentInfo* getComponentInfo() {return mpComponentInfo;}
   QList<ShapeAnnotation*> getShapesList() {return mShapesList;}
   QList<Component*> getInheritedComponentsList() {return mInheritedComponentsList;}
@@ -201,16 +217,28 @@ public:
   void setOldPosition(QPointF oldPosition) {mOldPosition = oldPosition;}
   QPointF getOldPosition() {return mOldPosition;}
   void setComponentFlags(bool enable);
-  QString getTransformationAnnotation();
-  QString getPlacementAnnotation();
+  QString getTransformationAnnotation(bool ModelicaSyntax);
+  QString getPlacementAnnotation(bool ModelicaSyntax = false);
   QString getOMCTransformationAnnotation(QPointF position);
   QString getOMCPlacementAnnotation(QPointF position);
   QString getTransformationOrigin();
   QString getTransformationExtent();
+  bool isExpandableConnector() const;
+  bool isArray() const;
+  int getArrayIndexAsNumber(bool *ok = 0) const;
+  bool isConnectorSizing();
+  static bool isParameterConnectorSizing(Component *pComponent, QString parameter);
   void createClassComponents();
   void applyRotation(qreal angle);
   void addConnectionDetails(LineAnnotation *pConnectorLineAnnotation);
   void removeConnectionDetails(LineAnnotation *pConnectorLineAnnotation);
+  void setHasTransition(bool hasTransition);
+  bool hasTransition() {return mHasTransition;}
+  void setIsInitialState(bool isInitialState);
+  bool isInitialState() {return mIsInitialState;}
+  void setActiveState(bool activeState) {mActiveState = activeState;}
+  bool isActiveState() {return mActiveState;}
+  void removeChildren();
   void emitAdded();
   void emitTransformChange() {emit transformChange();}
   void emitTransformHasChanged();
@@ -218,12 +246,21 @@ public:
   void emitDeleted();
   void componentParameterHasChanged();
   QString getParameterDisplayString(QString parameterName);
+  QString getDerivedClassModifierValue(QString modifierName);
+  QString getInheritedDerivedClassModifierValue(Component *pComponent, QString modifierName);
   void shapeAdded();
   void shapeUpdated();
   void shapeDeleted();
   void renameComponentInConnections(QString newName);
-  void insertInterfacePoint(QString interfaceName, QString position, QString angle321);
+  void insertInterfacePoint(QString interfaceName, QString position, QString angle321, int dimensions, QString causality, QString domain);
   void removeInterfacePoint(QString interfaceName);
+  void adjustInterfacePoints();
+  void updateComponentTransformations(const Transformation &oldTransformation);
+  void handleOMSComponentDoubleClick();
+  bool isInBus() {return mpBusComponent != 0;}
+  void setBusComponent(Component *pBusComponent);
+  Component* getBusComponent() {return mpBusComponent;}
+  Component* getComponentByName(const QString &componentName);
 
   Transformation mTransformation;
   Transformation mOldTransformation;
@@ -237,16 +274,18 @@ private:
   ComponentType mComponentType;
   QString mTransformationString;
   QStringList mDialogAnnotation;
+  QStringList mChoicesAnnotation;
   QString mParameterValue;
-  QGraphicsRectItem *mpResizerRectangle;
   LineAnnotation *mpNonExistingComponentLine;
   RectangleAnnotation *mpDefaultComponentRectangle;
   TextAnnotation *mpDefaultComponentText;
+  RectangleAnnotation *mpStateComponentRectangle;
   QAction *mpParametersAction;
+  QAction *mpFetchInterfaceDataAction;
   QAction *mpAttributesAction;
   QAction *mpOpenClassAction;
-  QAction *mpViewDocumentationAction;
   QAction *mpSubModelAttributesAction;
+  QAction *mpElementPropertiesAction;
   ResizerItem *mpBottomLeftResizerItem;
   ResizerItem *mpTopLeftResizerItem;
   ResizerItem *mpTopRightResizerItem;
@@ -264,15 +303,21 @@ private:
   QList<Component*> mComponentsList;
   QPointF mOldScenePosition;
   QPointF mOldPosition;
+  bool mHasTransition;
+  bool mIsInitialState;
+  bool mActiveState;
+  Component *mpBusComponent;
   void createNonExistingComponent();
   void createDefaultComponent();
+  void createStateComponent();
   void drawInterfacePoints();
   void drawComponent();
+  void drawModelicaComponent();
+  void drawOMSComponent();
   void drawInheritedComponentsAndShapes();
   void showNonExistingOrDefaultComponentIfNeeded();
   void createClassInheritedComponents();
   void createClassShapes();
-  void removeChildren();
   void createActions();
   void createResizerItems();
   void getResizerItemsPositions(qreal *x1, qreal *y1, qreal *x2, qreal *y2);
@@ -285,10 +330,12 @@ private:
   QString getParameterDisplayStringFromExtendsParameters(QString parameterName, QString modifierString);
   bool checkEnumerationDisplayString(QString &displayString, const QString &typeName);
   void updateToolTip();
+  bool canUseDiagramAnnotation();
 signals:
   void added();
   void transformChange();
   void transformHasChanged();
+  void transformChanging();
   void displayTextChanged();
   void changed();
   void deleted();
@@ -299,6 +346,7 @@ public slots:
   void handleUnloaded();
   void handleShapeAdded();
   void handleComponentAdded();
+  void handleNameChanged();
   void referenceComponentAdded();
   void referenceComponentTransformHasChanged();
   void referenceComponentChanged();
@@ -330,13 +378,13 @@ public slots:
   void moveCtrlRight();
   void showParameters();
   void showAttributes();
+  void fetchInterfaceData();
   void openClass();
-  void viewDocumentation();
   void showSubModelAttributes();
+  void showElementPropertiesDialog();
+  void updateDynamicSelect(double time);
 protected:
-  virtual void mouseDoubleClickEvent(QGraphicsSceneMouseEvent *event);
-  virtual void contextMenuEvent(QGraphicsSceneContextMenuEvent *event);
-  virtual QVariant itemChange(GraphicsItemChange change, const QVariant &value);
+  virtual QVariant itemChange(GraphicsItemChange change, const QVariant &value) override;
 };
 
 #endif // COMPONENT_H
