@@ -97,7 +97,6 @@ public:
   void undo();
 private:
   LibraryTreeItem *mpLibraryTreeItem;
-  StringHandler::ModelicaClasses mType;
   bool mAddObject;
   ComponentInfo *mpComponentInfo;
   Component *mpIconComponent;
@@ -124,15 +123,15 @@ private:
 class UpdateComponentAttributesCommand : public UndoCommand
 {
 public:
-  UpdateComponentAttributesCommand(Component *pComponent, const ComponentInfo &oldComponentInfo, const ComponentInfo &newComponentInfo,
-                                   bool duplicate = false, UndoCommand *pParent = 0);
+  UpdateComponentAttributesCommand(Component *pComponent, const ComponentInfo &oldComponentInfo, const ComponentInfo &newComponentInfo, UndoCommand *pParent = 0);
   void redoInternal();
   void undo();
+  static void updateComponentAttributes(Component *pComponent, const ComponentInfo &componentInfo);
+  static void updateComponentModifiers(Component *pComponent, const ComponentInfo &componentInfo);
 private:
   Component *mpComponent;
   ComponentInfo mOldComponentInfo;
   ComponentInfo mNewComponentInfo;
-  bool mDuplicate;
 };
 
 class UpdateComponentParametersCommand : public UndoCommand
@@ -185,6 +184,7 @@ public:
   UpdateConnectionCommand(LineAnnotation *pConnectionLineAnnotation, QString oldAnnotaton, QString newAnnotation, UndoCommand *pParent = 0);
   void redoInternal();
   void undo();
+  void redrawConnectionWithAnnotation(QString const& annotation);
 private:
   LineAnnotation *mpConnectionLineAnnotation;
   QString mOldAnnotation;
@@ -212,7 +212,6 @@ public:
   void undo();
 private:
   LineAnnotation *mpConnectionLineAnnotation;
-  GraphicsView *mpGraphicsView;
 };
 
 class AddTransitionCommand : public UndoCommand
@@ -235,6 +234,9 @@ public:
   void redoInternal();
   void undo();
 private:
+  void updateTransistionWithNewConditions();
+  void updateTransistionWithOldConditions();
+  void redrawTransitionWithUpdateFunction(const QString& annotation, std::function<void()> updateFunction);
   LineAnnotation *mpTransitionLineAnnotation;
   QString mOldCondition;
   bool mOldImmediate;
@@ -258,7 +260,6 @@ public:
   void undo();
 private:
   LineAnnotation *mpTransitionLineAnnotation;
-  GraphicsView *mpGraphicsView;
 };
 
 class AddInitialStateCommand : public UndoCommand
@@ -278,6 +279,7 @@ public:
   UpdateInitialStateCommand(LineAnnotation *pInitialStateLineAnnotation, QString oldAnnotaton, QString newAnnotation, UndoCommand *pParent = 0);
   void redoInternal();
   void undo();
+  void redrawInitialStateWithAnnotation(QString const& annotation);
 private:
   LineAnnotation *mpInitialStateLineAnnotation;
   QString mOldAnnotation;
@@ -292,7 +294,6 @@ public:
   void undo();
 private:
   LineAnnotation *mpInitialStateLineAnnotation;
-  GraphicsView *mpGraphicsView;
 };
 
 class UpdateCoOrdinateSystemCommand : public UndoCommand
@@ -300,8 +301,7 @@ class UpdateCoOrdinateSystemCommand : public UndoCommand
 public:
   UpdateCoOrdinateSystemCommand(GraphicsView *pGraphicsView, CoOrdinateSystem oldCoOrdinateSystem, CoOrdinateSystem newCoOrdinateSystem,
                                 bool copyProperties, QString oldVersion, QString newVersion, QString oldUsesAnnotationString,
-                                QString newUsesAnnotationString, QString oldOMCCommandLineOptions, QString newOMCCommandLineOptions,
-                                UndoCommand *pParent = 0);
+                                QString newUsesAnnotationString, UndoCommand *pParent = 0);
   void redoInternal();
   void undo();
 private:
@@ -313,8 +313,8 @@ private:
   QString mNewVersion;
   QString mOldUsesAnnotationString;
   QString mNewUsesAnnotationString;
-  QString mOldOMCCommandLineOptions;
-  QString mNewOMCCommandLineOptions;
+
+  void updateReferencedShapes(GraphicsView *pGraphicsView);
 };
 
 class UpdateClassAnnotationCommand : public UndoCommand
@@ -426,13 +426,14 @@ private:
 class AddSubModelCommand : public UndoCommand
 {
 public:
-  AddSubModelCommand(QString name, QString path, LibraryTreeItem *pLibraryTreeItem, QString annotation, bool openingClass,
+  AddSubModelCommand(QString name, QString path, QString startScript, LibraryTreeItem *pLibraryTreeItem, QString annotation, bool openingClass,
                      GraphicsView *pGraphicsView, UndoCommand *pParent = 0);
   void redoInternal();
   void undo();
 private:
   QString mName;
   QString mPath;
+  QString mStartScript;
   LibraryTreeItem *mpLibraryTreeItem;
   QString mAnnotation;
   bool mOpeningClass;
@@ -630,29 +631,27 @@ private:
 class UpdateTLMParametersCommand : public UndoCommand
 {
 public:
-  UpdateTLMParametersCommand(LineAnnotation *pConnectionLineAnnotation, const oms3_tlm_connection_parameters_t oldTLMParameters,
-                             const oms3_tlm_connection_parameters_t newTLMParameters, UndoCommand *pParent = 0);
+  UpdateTLMParametersCommand(LineAnnotation *pConnectionLineAnnotation, const oms_tlm_connection_parameters_t oldTLMParameters,
+                             const oms_tlm_connection_parameters_t newTLMParameters, UndoCommand *pParent = 0);
   void redoInternal();
   void undo();
 private:
   LineAnnotation *mpConnectionLineAnnotation;
-  oms3_tlm_connection_parameters_t mOldTLMParameters;
-  oms3_tlm_connection_parameters_t mNewTLMParameters;
+  oms_tlm_connection_parameters_t mOldTLMParameters;
+  oms_tlm_connection_parameters_t mNewTLMParameters;
 };
 
 class SystemSimulationInformationCommand : public UndoCommand
 {
 public:
   SystemSimulationInformationCommand(TLMSystemSimulationInformation *pTLMSystemSimulationInformation,
-                                     WCSystemSimulationInformation *pWCSystemSimulationInformation,
-                                     SCSystemSimulationInformation *pSCSystemSimulationInformation,
+                                     WCSCSystemSimulationInformation *pWCSCSystemSimulationInformation,
                                      LibraryTreeItem *pLibraryTreeItem, UndoCommand *pParent = 0);
   void redoInternal();
   void undo();
 private:
   TLMSystemSimulationInformation *mpTLMSystemSimulationInformation;
-  WCSystemSimulationInformation *mpWCSystemSimulationInformation;
-  SCSystemSimulationInformation *mpSCSystemSimulationInformation;
+  WCSCSystemSimulationInformation *mpWCSCSystemSimulationInformation;
   LibraryTreeItem *mpLibraryTreeItem;
 };
 
