@@ -1,7 +1,7 @@
 /*
  * This file is part of OpenModelica.
  *
- * Copyright (c) 1998-2014, Open Source Modelica Consortium (OSMC),
+ * Copyright (c) 1998-CurrentYear, Open Source Modelica Consortium (OSMC),
  * c/o Linköpings universitet, Department of Computer and Information Science,
  * SE-58183 Linköping, Sweden.
  *
@@ -29,11 +29,7 @@
  *
  */
 /*
- *
  * @author Adeel Asghar <adeel.asghar@liu.se>
- *
- * RCS: $Id$
- *
  */
 
 //! @brief Contains functions used for parsing results obtained from OpenModelica Compiler.
@@ -41,6 +37,12 @@
 #include "StringHandler.h"
 #include "Helper.h"
 #include "Utilities.h"
+#include "Util/ResourceCache.h"
+
+#include <QtCore/qmath.h>
+#include <QDir>
+#include <QFileDialog>
+#include <QTextCodec>
 
 #if (QT_VERSION >= QT_VERSION_CHECK(5, 0, 0))
 #define toAscii toLatin1
@@ -64,6 +66,50 @@ StringHandler::~StringHandler()
 
 }
 
+/*!
+ * \brief StringHandler::getTLMCausality
+ * Returns the TLM causality as string.
+ * \param causality
+ * \return
+ */
+QString StringHandler::getTLMCausality(int causality)
+{
+  switch (causality) {
+    case TLMInput:
+      return "Input";
+    case TLMOutput:
+      return "Output";
+    case TLMBidirectional:
+    default:
+      return "Bidirectional";
+  }
+}
+
+/*!
+ * \brief StringHandler::getTLMDomain
+ * Returns the TLM domain as string.
+ * \param domain
+ * \return
+ */
+QString StringHandler::getTLMDomain(int domain)
+{
+  switch (domain) {
+    case Mechanical:
+      return "Mechanical";
+    case Electric:
+      return "Electric";
+    case Hydraulic:
+      return "Hydraulic";
+    case Pneumatic:
+      return "Pneumatic";
+    case Magnetic:
+      return "Magnetic";
+    default:
+      //Should never be reahed
+      return "";
+  }
+}
+
 QString StringHandler::getSimulationTool(int tool)
 {
   switch (tool)
@@ -383,12 +429,12 @@ QComboBox* StringHandler::getLinePatternComboBox()
 {
   QComboBox *pLinePatternComboBox = new QComboBox;
   pLinePatternComboBox->setIconSize(QSize(58, 16));
-  pLinePatternComboBox->addItem(QIcon(":/Resources/icons/line-none.svg"), getLinePatternString(LineNone));
-  pLinePatternComboBox->addItem(QIcon(":/Resources/icons/line-solid.svg"), getLinePatternString(LineSolid));
-  pLinePatternComboBox->addItem(QIcon(":/Resources/icons/line-dash.svg"), getLinePatternString(LineDash));
-  pLinePatternComboBox->addItem(QIcon(":/Resources/icons/line-dot.svg"), getLinePatternString(LineDot));
-  pLinePatternComboBox->addItem(QIcon(":/Resources/icons/line-dash-dot.svg"), getLinePatternString(LineDashDot));
-  pLinePatternComboBox->addItem(QIcon(":/Resources/icons/line-dash-dot-dot.svg"), getLinePatternString(LineDashDotDot));
+  pLinePatternComboBox->addItem(ResourceCache::getIcon(":/Resources/icons/line-none.svg"), getLinePatternString(LineNone));
+  pLinePatternComboBox->addItem(ResourceCache::getIcon(":/Resources/icons/line-solid.svg"), getLinePatternString(LineSolid));
+  pLinePatternComboBox->addItem(ResourceCache::getIcon(":/Resources/icons/line-dash.svg"), getLinePatternString(LineDash));
+  pLinePatternComboBox->addItem(ResourceCache::getIcon(":/Resources/icons/line-dot.svg"), getLinePatternString(LineDot));
+  pLinePatternComboBox->addItem(ResourceCache::getIcon(":/Resources/icons/line-dash-dot.svg"), getLinePatternString(LineDashDot));
+  pLinePatternComboBox->addItem(ResourceCache::getIcon(":/Resources/icons/line-dash-dot-dot.svg"), getLinePatternString(LineDashDotDot));
   return pLinePatternComboBox;
 }
 
@@ -487,17 +533,17 @@ QString StringHandler::getFillPatternString(StringHandler::FillPattern type)
 QComboBox* StringHandler::getFillPatternComboBox()
 {
   QComboBox *pFillPatternComboBox = new QComboBox;
-  pFillPatternComboBox->addItem(QIcon(":/Resources/icons/fill-none.svg"), getFillPatternString(FillNone));
-  pFillPatternComboBox->addItem(QIcon(":/Resources/icons/fill-solid.svg"), getFillPatternString(FillSolid));
-  pFillPatternComboBox->addItem(QIcon(":/Resources/icons/fill-horizontal.svg"), getFillPatternString(FillHorizontal));
-  pFillPatternComboBox->addItem(QIcon(":/Resources/icons/fill-vertical.svg"), getFillPatternString(FillVertical));
-  pFillPatternComboBox->addItem(QIcon(":/Resources/icons/fill-cross.svg"), getFillPatternString(FillCross));
-  pFillPatternComboBox->addItem(QIcon(":/Resources/icons/fill-forward.svg"), getFillPatternString(FillForward));
-  pFillPatternComboBox->addItem(QIcon(":/Resources/icons/fill-backward.svg"), getFillPatternString(FillBackward));
-  pFillPatternComboBox->addItem(QIcon(":/Resources/icons/fill-cross-diagnol.svg"), getFillPatternString(FillCrossDiag));
-  pFillPatternComboBox->addItem(QIcon(":/Resources/icons/fill-horizontal-cylinder.svg"), getFillPatternString(FillHorizontalCylinder));
-  pFillPatternComboBox->addItem(QIcon(":/Resources/icons/fill-vertical-cylinder.svg"), getFillPatternString(FillVerticalCylinder));
-  pFillPatternComboBox->addItem(QIcon(":/Resources/icons/fill-sphere.svg"), getFillPatternString(FillSphere));
+  pFillPatternComboBox->addItem(ResourceCache::getIcon(":/Resources/icons/fill-none.svg"), getFillPatternString(FillNone));
+  pFillPatternComboBox->addItem(ResourceCache::getIcon(":/Resources/icons/fill-solid.svg"), getFillPatternString(FillSolid));
+  pFillPatternComboBox->addItem(ResourceCache::getIcon(":/Resources/icons/fill-horizontal.svg"), getFillPatternString(FillHorizontal));
+  pFillPatternComboBox->addItem(ResourceCache::getIcon(":/Resources/icons/fill-vertical.svg"), getFillPatternString(FillVertical));
+  pFillPatternComboBox->addItem(ResourceCache::getIcon(":/Resources/icons/fill-cross.svg"), getFillPatternString(FillCross));
+  pFillPatternComboBox->addItem(ResourceCache::getIcon(":/Resources/icons/fill-forward.svg"), getFillPatternString(FillForward));
+  pFillPatternComboBox->addItem(ResourceCache::getIcon(":/Resources/icons/fill-backward.svg"), getFillPatternString(FillBackward));
+  pFillPatternComboBox->addItem(ResourceCache::getIcon(":/Resources/icons/fill-cross-diagnol.svg"), getFillPatternString(FillCrossDiag));
+  pFillPatternComboBox->addItem(ResourceCache::getIcon(":/Resources/icons/fill-horizontal-cylinder.svg"), getFillPatternString(FillHorizontalCylinder));
+  pFillPatternComboBox->addItem(ResourceCache::getIcon(":/Resources/icons/fill-vertical-cylinder.svg"), getFillPatternString(FillVerticalCylinder));
+  pFillPatternComboBox->addItem(ResourceCache::getIcon(":/Resources/icons/fill-sphere.svg"), getFillPatternString(FillSphere));
   return pFillPatternComboBox;
 }
 
@@ -592,10 +638,10 @@ QComboBox* StringHandler::getStartArrowComboBox()
 {
   QComboBox *pStartArrowComboBox = new QComboBox;
   pStartArrowComboBox->setIconSize(QSize(58, 16));
-  pStartArrowComboBox->addItem(QIcon(":/Resources/icons/line-solid.svg"), getArrowString(ArrowNone));
-  pStartArrowComboBox->addItem(QIcon(":/Resources/icons/arrow-start-open.svg"), getArrowString(ArrowOpen));
-  pStartArrowComboBox->addItem(QIcon(":/Resources/icons/arrow-start-fill.svg"), getArrowString(ArrowFilled));
-  pStartArrowComboBox->addItem(QIcon(":/Resources/icons/arrow-start-open-half.svg"), getArrowString(ArrowHalf));
+  pStartArrowComboBox->addItem(ResourceCache::getIcon(":/Resources/icons/line-solid.svg"), getArrowString(ArrowNone));
+  pStartArrowComboBox->addItem(ResourceCache::getIcon(":/Resources/icons/arrow-start-open.svg"), getArrowString(ArrowOpen));
+  pStartArrowComboBox->addItem(ResourceCache::getIcon(":/Resources/icons/arrow-start-fill.svg"), getArrowString(ArrowFilled));
+  pStartArrowComboBox->addItem(ResourceCache::getIcon(":/Resources/icons/arrow-start-open-half.svg"), getArrowString(ArrowHalf));
   return pStartArrowComboBox;
 }
 
@@ -603,47 +649,70 @@ QComboBox* StringHandler::getEndArrowComboBox()
 {
   QComboBox *pEndArrowComboBox = new QComboBox;
   pEndArrowComboBox->setIconSize(QSize(58, 16));
-  pEndArrowComboBox->addItem(QIcon(":/Resources/icons/line-solid.svg"), getArrowString(ArrowNone));
-  pEndArrowComboBox->addItem(QIcon(":/Resources/icons/arrow-end-open.svg"), getArrowString(ArrowOpen));
-  pEndArrowComboBox->addItem(QIcon(":/Resources/icons/arrow-end-fill.svg"), getArrowString(ArrowFilled));
-  pEndArrowComboBox->addItem(QIcon(":/Resources/icons/arrow-end-open-half.svg"), getArrowString(ArrowHalf));
+  pEndArrowComboBox->addItem(ResourceCache::getIcon(":/Resources/icons/line-solid.svg"), getArrowString(ArrowNone));
+  pEndArrowComboBox->addItem(ResourceCache::getIcon(":/Resources/icons/arrow-end-open.svg"), getArrowString(ArrowOpen));
+  pEndArrowComboBox->addItem(ResourceCache::getIcon(":/Resources/icons/arrow-end-fill.svg"), getArrowString(ArrowFilled));
+  pEndArrowComboBox->addItem(ResourceCache::getIcon(":/Resources/icons/arrow-end-open-half.svg"), getArrowString(ArrowHalf));
   return pEndArrowComboBox;
 }
 
+/*!
+ * \brief StringHandler::getFontWeight
+ * \param styleList
+ * Returns the font weight
+ * \return
+ */
 int StringHandler::getFontWeight(QList<StringHandler::TextStyle> styleList)
 {
-  foreach (StringHandler::TextStyle textStyle, styleList)
-  {
-    if (textStyle == StringHandler::TextStyleBold)
+  foreach (StringHandler::TextStyle textStyle, styleList) {
+    if (textStyle == StringHandler::TextStyleBold) {
       return QFont::Bold;
+    }
   }
   return QFont::Normal;
 }
 
+/*!
+ * \brief StringHandler::getFontItalic
+ * \param styleList
+ * Returns true if font is italic.
+ * \return
+ */
 bool StringHandler::getFontItalic(QList<StringHandler::TextStyle> styleList)
 {
-  foreach (StringHandler::TextStyle textStyle, styleList)
-  {
-    if (textStyle == StringHandler::TextStyleItalic)
+  foreach (StringHandler::TextStyle textStyle, styleList) {
+    if (textStyle == StringHandler::TextStyleItalic) {
       return true;
+    }
   }
   return false;
 }
 
+/*!
+ * \brief StringHandler::getFontUnderline
+ * \param styleList
+ * Returns true is font is underline.
+ * \return
+ */
 bool StringHandler::getFontUnderline(QList<StringHandler::TextStyle> styleList)
 {
-  foreach (StringHandler::TextStyle textStyle, styleList)
-  {
-    if (textStyle == StringHandler::TextStyleUnderLine)
+  foreach (StringHandler::TextStyle textStyle, styleList) {
+    if (textStyle == StringHandler::TextStyleUnderLine) {
       return true;
+    }
   }
   return false;
 }
 
+/*!
+ * \brief StringHandler::getTextAlignment
+ * \param alignment
+ * Returns the text alignment
+ * \return
+ */
 Qt::Alignment StringHandler::getTextAlignment(StringHandler::TextAlignment alignment)
 {
-  switch (alignment)
-  {
+  switch (alignment) {
     case StringHandler::TextAlignmentLeft:
       return Qt::AlignLeft;
     case StringHandler::TextAlignmentCenter:
@@ -655,22 +724,34 @@ Qt::Alignment StringHandler::getTextAlignment(StringHandler::TextAlignment align
   }
 }
 
+/*!
+ * \brief StringHandler::getTextAlignmentType
+ * \param alignment
+ * Returns the text alignment type.
+ * \return
+ */
 StringHandler::TextAlignment StringHandler::getTextAlignmentType(QString alignment)
 {
-  if (alignment.compare("TextAlignment.Left") == 0)
+  if (alignment.compare("TextAlignment.Left") == 0) {
     return StringHandler::TextAlignmentLeft;
-  else if (alignment.compare("TextAlignment.Center") == 0)
+  } else if (alignment.compare("TextAlignment.Center") == 0) {
     return StringHandler::TextAlignmentCenter;
-  else if (alignment.compare("TextAlignment.Right") == 0)
+  } else if (alignment.compare("TextAlignment.Right") == 0) {
     return StringHandler::TextAlignmentRight;
-  else
+  } else {
     return StringHandler::TextAlignmentCenter;
+  }
 }
 
+/*!
+ * \brief StringHandler::getTextAlignmentString
+ * \param alignment
+ * Returns the text alignment as string.
+ * \return
+ */
 QString StringHandler::getTextAlignmentString(StringHandler::TextAlignment alignment)
 {
-  switch (alignment)
-  {
+  switch (alignment) {
     case StringHandler::TextAlignmentLeft:
       return "TextAlignment.Left";
     case StringHandler::TextAlignmentCenter:
@@ -682,10 +763,15 @@ QString StringHandler::getTextAlignmentString(StringHandler::TextAlignment align
   }
 }
 
+/*!
+ * \brief StringHandler::getTextStyleString
+ * \param textStyle
+ * Returns the text syle.
+ * \return
+ */
 QString StringHandler::getTextStyleString(StringHandler::TextStyle textStyle)
 {
-  switch (textStyle)
-  {
+  switch (textStyle) {
     case StringHandler::TextStyleBold:
       return "TextStyle.Bold";
     case StringHandler::TextStyleItalic:
@@ -697,25 +783,46 @@ QString StringHandler::getTextStyleString(StringHandler::TextStyle textStyle)
   }
 }
 
-//! Removes the first and last curly brackest {} from the string.
-//! @param value is the string which is parsed.
+/*!
+ * \brief StringHandler::removeFirstLastCurlBrackets
+ * Removes the first and last curly brackets {} from the string.
+ * \param value is the string which is parsed.
+ * \return
+ */
 QString StringHandler::removeFirstLastCurlBrackets(QString value)
 {
   value = value.trimmed();
-  if (value.length() > 1 && value.at(0) == '{' && value.at(value.length() - 1) == '}')
-  {
+  if (value.length() > 1 && value.at(0) == '{' && value.at(value.length() - 1) == '}') {
     value = value.mid(1, (value.length() - 2));
   }
   return value;
 }
 
-//! Removes the first and last brackest () from the string.
-//! @param value is the string which is parsed.
-QString StringHandler::removeFirstLastBrackets(QString value)
+/*!
+ * \brief StringHandler::removeFirstLastParentheses
+ * Removes the first and last parentheses () from the string.
+ * \param value is the string which is parsed.
+ * \return
+ */
+QString StringHandler::removeFirstLastParentheses(QString value)
 {
   value = value.trimmed();
-  if (value.length() > 1 && value.at(0) == '(' && value.at(value.length() - 1) == ')')
-  {
+  if (value.length() > 1 && value.at(0) == '(' && value.at(value.length() - 1) == ')') {
+    value = value.mid(1, (value.length() - 2));
+  }
+  return value;
+}
+
+/*!
+ * \brief StringHandler::removeFirstLastSquareBrackets
+ * Removes the first and last sqaure brackets [] from the string.
+ * \param value is the string which is parsed.
+ * \return
+ */
+QString StringHandler::removeFirstLastSquareBrackets(QString value)
+{
+  value = value.trimmed();
+  if (value.length() > 1 && value.at(0) == '[' && value.at(value.length() - 1) == ']') {
     value = value.mid(1, (value.length() - 2));
   }
   return value;
@@ -726,28 +833,25 @@ QString StringHandler::removeFirstLastBrackets(QString value)
 QString StringHandler::removeFirstLastQuotes(QString value)
 {
   value = value.trimmed();
-  if (value.length() > 1 && value.at(0) == '\"' && value.at(value.length() - 1) == '\"')
-  {
+  if (value.length() > 1 && value.at(0) == '\"' && value.at(value.length() - 1) == '\"') {
     value = value.mid(1, (value.length() - 2));
   }
   return value;
 }
 
 /*!
- * \brief StringHandler::getSubStringBeforeDots
- * Returns the first word from a string.
- * \param value -  is the string which is parsed.
+ * \brief StringHandler::removeFirstLastSingleQuotes
+ * Removes the first and last single quotes from the string.
+ * \param value
  * \return
  */
-QString StringHandler::getSubStringBeforeDots(QString value)
+QString StringHandler::removeFirstLastSingleQuotes(QString value)
 {
-  if (value.isEmpty())
-  {
-    return "";
-  }
   value = value.trimmed();
-  QStringList list = value.split(".", QString::SkipEmptyParts);
-  return list.at(0);
+  if (value.length() > 1 && value.at(0) == '\'' && value.at(value.length() - 1) == '\'') {
+    value = value.mid(1, (value.length() - 2));
+  }
+  return value;
 }
 
 QStringList StringHandler::getStrings(QString value)
@@ -936,24 +1040,6 @@ QString StringHandler::removeFirstWordAfterDot(QString value)
   return wordsBeforeAfterFirstDot(value, false);
 }
 
-QString StringHandler::getModifierValue(QString value)
-{
-  int element = 0;
-  for(int i = 0 ; i < value.length() ; i++)
-  {
-    if (value.at(i) == '(')
-      element++;
-    else if (value.at(i) == ')')
-      element--;
-    else if (value.at(i) == '=')
-    {
-      if (element == 0)
-        return value.mid(i + 1);
-    }
-  }
-  return "";
-}
-
 QString StringHandler::escapeString(QString value)
 {
   QString res;
@@ -976,6 +1062,20 @@ QString StringHandler::escapeString(QString value)
   return res;
 }
 
+QString StringHandler::escapeStringQuotes(QString value)
+{
+  QString res;
+  value = value.trimmed();
+  for (int i = 0; i < value.length(); i++) {
+    switch (value[i].toAscii())
+    {
+      case '"':  res.append("\\\"");   break;
+      default:   res.append(value[i]); break;
+    }
+  }
+  return res;
+}
+
 #define CONSUME_CHAR(value,res,i) \
   if (value.at(i) == '\\') { \
   i++; \
@@ -1046,7 +1146,7 @@ QStringList StringHandler::unparseStrings(QString value)
         i++;
       continue;
     }
-    while (value[i] != '"' && value[i] != '\0') {
+    while (value[i] != '"' && !value[i].isNull()) {
       i++;
       fprintf(stderr, "error? malformed string-list. skipping: %c\n", value[i].toAscii());
     }
@@ -1125,9 +1225,13 @@ QString StringHandler::getSaveFileName(QWidget* parent, const QString &caption,
   /* Add the extension with purposedName because if the directory with the same name exists then
    * QFileDialog::getSaveFileName takes the user to that directory and does not show the purposedName.
    */
-  QString proposedFileName = *proposedName;
-  if (!proposedFileName.isEmpty() && !defaultSuffix.isEmpty())
-    proposedFileName = QString(proposedFileName).append(".").append(defaultSuffix);
+  QString proposedFileName = "";
+  if (proposedName) {
+    proposedFileName = *proposedName;
+    if (!proposedFileName.isEmpty() && !defaultSuffix.isEmpty()) {
+      proposedFileName = QString(proposedFileName).append(".").append(defaultSuffix);
+    }
+  }
 
   if (!proposedFileName.isEmpty()) {
     fileName = QFileDialog::getSaveFileName(parent, caption, QString(dir_str).append("/").append(proposedFileName), filter, selectedFilter);
@@ -1192,7 +1296,7 @@ QString StringHandler::getOpenFileName(QWidget* parent, const QString &caption,
   QFileDialog *dialog;
   dialog = new QFileDialog(parent, caption, dir_str, filter);
   QList<QUrl> urls = dialog->sidebarUrls();
-  urls << QUrl("file://" + OpenModelica::tempDirectory());
+  urls << QUrl("file://" + Utilities::tempDirectory());
   dialog->setSidebarUrls(urls);
   dialog->setFileMode(QFileDialog::ExistingFile);
   if (dialog->exec()) {
@@ -1225,7 +1329,7 @@ QStringList StringHandler::getOpenFileNames(QWidget* parent, const QString &capt
   QFileDialog *dialog;
   dialog = new QFileDialog(parent, caption, dir_str, filter);
   QList<QUrl> urls = dialog->sidebarUrls();
-  urls << QUrl("file://" + OpenModelica::tempDirectory());
+  urls << QUrl("file://" + Utilities::tempDirectory());
   dialog->setSidebarUrls(urls);
   dialog->setFileMode(QFileDialog::ExistingFiles);
   if (dialog->exec()) {
@@ -1268,21 +1372,22 @@ QString StringHandler::getLastOpenDirectory()
   return mLastOpenDir;
 }
 
-QStringList StringHandler::getDialogAnnotation(QString componentAnnotation)
+QStringList StringHandler::getAnnotation(QString componentAnnotation, QString annotationName)
 {
-  if (componentAnnotation.toLower().contains("error")) {
-    return QStringList();
-  }
   componentAnnotation = StringHandler::removeFirstLastCurlBrackets(componentAnnotation);
   if (componentAnnotation.isEmpty()) {
     return QStringList();
   }
   QStringList annotations = StringHandler::getStrings(componentAnnotation, '(', ')');
   foreach (QString annotation, annotations) {
-    if (annotation.startsWith("Dialog")) {
-      annotation = annotation.mid(QString("Dialog").length());
-      annotation = StringHandler::removeFirstLastBrackets(annotation);
-      return StringHandler::getStrings(annotation);
+    if (annotation.startsWith(annotationName)) {
+      annotation = annotation.mid(QString(annotationName).length());
+      annotation = StringHandler::removeFirstLastParentheses(annotation);
+      if (annotation.toLower().contains("error")) {
+        return QStringList();
+      } else {
+        return StringHandler::getStrings(annotation);
+      }
     }
   }
   return QStringList();
@@ -1290,9 +1395,6 @@ QStringList StringHandler::getDialogAnnotation(QString componentAnnotation)
 
 QString StringHandler::getPlacementAnnotation(QString componentAnnotation)
 {
-  if (componentAnnotation.toLower().contains("error")) {
-    return "";
-  }
   componentAnnotation = StringHandler::removeFirstLastCurlBrackets(componentAnnotation);
   if (componentAnnotation.isEmpty()) {
     return "";
@@ -1300,7 +1402,12 @@ QString StringHandler::getPlacementAnnotation(QString componentAnnotation)
   QStringList annotations = StringHandler::getStrings(componentAnnotation, '(', ')');
   foreach (QString annotation, annotations) {
     if (annotation.startsWith("Placement")) {
-      return StringHandler::removeFirstLastBrackets(annotation);
+      QString placementAnnotation = StringHandler::removeFirstLastParentheses(annotation);
+      if (placementAnnotation.toLower().contains("error")) {
+        return "";
+      } else {
+        return placementAnnotation;
+      }
     }
   }
   return "";
@@ -1402,25 +1509,27 @@ QStringList StringHandler::makeVariableParts(QString variable)
   return variable.split(QRegExp("\\.(?![^\\[\\]]*\\])"), QString::SkipEmptyParts);
 }
 
-bool StringHandler::isCFile(QString extension)
+#include <iostream>
+using namespace std;
+
+QStringList StringHandler::makeVariablePartsWithInd(QString variable)
 {
-  if (extension.compare("c") == 0 ||
-      extension.compare("cpp") == 0 ||
-      extension.compare("cc") == 0 ||
-      extension.compare("h") == 0 ||
-      extension.compare("hpp") == 0)
-    return true;
-  else
-    return false;
+  QStringList varParts = makeVariableParts(variable);
+  //if the last part is array with index, split it into the name and index parts:
+
+  if (!varParts.isEmpty()) {
+	  QString* lastStr = &(varParts.last());
+	  int i = lastStr->lastIndexOf(QRegExp("\\[\\d+\\]"));
+	  if(i>=0){
+		  QString indexPart = *lastStr;
+		  indexPart.remove(0,i);
+		  lastStr->truncate(i);
+		  varParts.append(indexPart);
+	  }
+  }
+  return varParts;
 }
 
-bool StringHandler::isModelicaFile(QString extension)
-{
-  if (extension.compare("mo") == 0)
-    return true;
-  else
-    return false;
-}
 
 bool StringHandler::naturalSort(const QString &s1, const QString &s2) {
   int i1 = 0; // index in string
@@ -1478,7 +1587,7 @@ QProcessEnvironment StringHandler::simulationProcessEnvironment()
 {
   QProcessEnvironment environment = QProcessEnvironment::systemEnvironment();
   QString OMHOME = QString(Helper::OpenModelicaHome).replace("/", "\\");
-  QString OMHOMEBin = OMHOME + "\\bin;" + OMHOME + "\\lib\\omc\\msvc;" + OMHOME + "\\lib\\omc\\cpp\\msvc";
+  QString OMHOMEBin = OMHOME + "\\bin;" + OMHOME + "\\lib\\omc\\msvc;" + OMHOME + "\\lib\\omc\\cpp;" + OMHOME + "\\lib\\omc\\cpp\\msvc";
   environment.insert("PATH", OMHOMEBin + ";" + environment.value("PATH"));
   return environment;
 }
@@ -1531,8 +1640,8 @@ QColor StringHandler::getSimulationMessageTypeColor(StringHandler::SimulationMes
     case StringHandler::SMWarning:
     case StringHandler::Error:
     case StringHandler::Assert:
-    case StringHandler::Debug:
       return Qt::red;
+    case StringHandler::Debug:
     case StringHandler::Info:
     case StringHandler::Unknown:
     default:
@@ -1565,17 +1674,47 @@ QString StringHandler::makeClassNameRelative(QString draggedClassName, QString d
  */
 QString StringHandler::toCamelCase(QString str)
 {
+  // if the name is all caps then convert it to lower and return it.
+  if (str.toUpper().compare(str, Qt::CaseSensitive) == 0) {
+    return str.toLower();
+  }
   QString s = str;
   s[0] = s[0].toLower();
   return s;
 }
 
 /*!
- * \brief StringHandler::getTrailingSpacesSize
- * \param str
- * \return the number of trailing spaces in a string.
+ * \brief StringHandler::getLeadingSpaces
+ * Returns a map with line number and number of leading spaces in that line.
+ * \param contents
+ * \return
  */
-int StringHandler::getTrailingSpacesSize(QString str)
+QMap<int, int> StringHandler::getLeadingSpaces(QString contents)
+{
+  QMap<int, int> leadingSpacesMap;
+  int startLeadingSpaces, leadingSpaces = 0;
+  QTextStream textStream(&contents);
+  int lineNumber = 1;
+  while (!textStream.atEnd()) {
+    QString currentLine = textStream.readLine();
+    if (lineNumber == 1) {  // the first line
+      startLeadingSpaces = StringHandler::getLeadingSpacesSize(currentLine);
+      leadingSpaces = startLeadingSpaces;
+    } else {
+      leadingSpaces = qMin(startLeadingSpaces, StringHandler::getLeadingSpacesSize(currentLine));
+    }
+    leadingSpacesMap.insert(lineNumber, leadingSpaces);
+    lineNumber++;
+  }
+  return leadingSpacesMap;
+}
+
+/*!
+ * \brief StringHandler::getLeadingSpacesSize
+ * \param str
+ * \return the number of leading spaces in a string.
+ */
+int StringHandler::getLeadingSpacesSize(QString str)
 {
   int i = 0;
   while (i < str.size()) {
@@ -1602,3 +1741,118 @@ bool StringHandler::isFileWritAble(QString filePath)
     return true;
   }
 }
+
+/*!
+ * \brief StringHandler::containsSpace
+ * Returns true if string contains a space.
+ * \param str
+ * \return
+ */
+bool StringHandler::containsSpace(QString str)
+{
+  for (int i = 0 ; i < str.size() ; i++) {
+    if (str.at(i).isSpace()) {
+      return true;
+    }
+  }
+  return false;
+}
+
+/*!
+ * \brief StringHandler::trimmedEnd
+ * Trims the whitespace from the end of the string.
+ * \param str
+ * \return
+ */
+QString StringHandler::trimmedEnd(const QString &str)
+{
+  int n = str.size() - 1;
+  for (; n >= 0; --n) {
+    if (!str.at(n).isSpace()) {
+      return str.left(n + 1);
+    }
+  }
+  return "";
+}
+
+/*!
+ * \brief StringHandler::joinDerivativeAndPreviousVariable
+ * Joins the variable. For example, if we have variable like der(der(mass.flange_a.s)) we need to display der(der(s)).
+ * \param fullVariableName
+ * \param variableName
+ * \param derivativeOrPrevious
+ * \return
+ */
+QString StringHandler::joinDerivativeAndPreviousVariable(QString fullVariableName, QString variableName, QString derivativeOrPrevious)
+{
+  int times = (fullVariableName.lastIndexOf(derivativeOrPrevious) / derivativeOrPrevious.size()) + 1;
+  return QString("%1%2%3").arg(QString(derivativeOrPrevious).repeated(times), variableName, QString(")").repeated(times));
+}
+
+/*!
+ * \brief StringHandler::removeLeadingSpaces
+ * Removes the leading spaces from a nested class text to make it more readable.
+ * \param contents
+ * \return
+ */
+QString StringHandler::removeLeadingSpaces(QString contents)
+{
+  QString text;
+  int startLeadingSpaces = 0;
+  int leadingSpaces = 0;
+  QTextStream textStream(&contents);
+  int lineNumber = 1;
+  while (!textStream.atEnd()) {
+    QString currentLine = textStream.readLine();
+    if (lineNumber == 1) {  // the first line
+      startLeadingSpaces = StringHandler::getLeadingSpacesSize(currentLine);
+      leadingSpaces = startLeadingSpaces;
+    } else {
+      leadingSpaces = qMin(startLeadingSpaces, StringHandler::getLeadingSpacesSize(currentLine));
+    }
+    text += currentLine.mid(leadingSpaces) + "\n";
+    lineNumber++;
+  }
+  return text;
+}
+
+QString StringHandler::removeLine(QString text, QString lineToRemove)
+{
+  QString classText;
+  QTextStream textStream(&text);
+  while (!textStream.atEnd()) {
+    QString currentLine = textStream.readLine();
+    if (currentLine.compare(lineToRemove) != 0) {
+      classText += currentLine + "\n";
+    }
+  }
+  return classText;
+}
+
+/*!
+ * \brief StringHandler::insertClassAtPosition
+ * Inserts the childClassText inside a parentClassText at linePosition.
+ * \param parentClassText
+ * \param childClassText
+ * \param linePosition
+ * \param nestedLevel
+ * \return
+ */
+QString StringHandler::insertClassAtPosition(QString parentClassText, QString childClassText, int linePosition, int nestedLevel)
+{
+  QString classText;
+  QTextStream parentTextStream(&parentClassText);
+  int lineNumber = 1;
+  while (!parentTextStream.atEnd()) {
+    QString currentLine = parentTextStream.readLine();
+    classText += currentLine + "\n";
+    if (linePosition == lineNumber) {
+      QTextStream childTextStream(&childClassText);
+      while (!childTextStream.atEnd()) {
+        classText += QString(' ').repeated(nestedLevel) + childTextStream.readLine() + "\n";
+      }
+    }
+    lineNumber++;
+  }
+  return classText;
+}
